@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hcid.h,v 1.10 2004/08/09 08:01:36 holtmann Exp $
+ *  $Id: hcid.h,v 1.11 2004/11/14 02:17:58 holtmann Exp $
  */
 
 #include <sys/types.h>
@@ -97,7 +97,7 @@ extern struct hcid_opts hcid;
 
 int read_config(char *file);
 
-struct device_opts *alloc_device_opts(char *addr);
+struct device_opts *alloc_device_opts(char *ref);
 
 gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer data);
 gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data);
