@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: l2test.c,v 1.11 2004/04/28 12:44:33 holtmann Exp $
+ *  $Id: l2test.c,v 1.16 2005/01/11 21:42:38 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,27 +33,21 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <syslog.h>
-#include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <syslog.h>
 #include <signal.h>
 #include <sys/time.h>
-#include <sys/select.h>
-
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <resolv.h>
-#include <netdb.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
+#include <bluetooth/hci.h>
+#include <bluetooth/hci_lib.h>
 #include <bluetooth/l2cap.h>
 
-#define NIBBLE_TO_ASCII(c)      ((c) < 0x0a ? (c) + 0x30 : (c) + 0x57)
+#define NIBBLE_TO_ASCII(c)  ((c) < 0x0a ? (c) + 0x30 : (c) + 0x57)
 
 /* Test modes */
 enum {
@@ -69,35 +63,36 @@ enum {
 	LSENDDUMP
 };
 
-unsigned char *buf;
+static unsigned char *buf;
 
 /* Default mtu */
-int imtu = 672;
-int omtu = 0;
+static int imtu = 672;
+static int omtu = 0;
 
 /* Default data size */
-long data_size = 672;
+static long data_size = 672;
 
 /* Default addr and psm */
-bdaddr_t bdaddr;
-unsigned short psm = 10;
+static bdaddr_t bdaddr;
+static unsigned short psm = 10;
 
 /* Default number of frames to send */
-int num_frames = -1; // Infinite
+static int num_frames = -1; // Infinite
 
-int master = 0;
-int auth = 0;
-int encrypt = 0;
-int socktype = SOCK_SEQPACKET;
-int linger = 0;
-int reliable = 0;
+static int master = 0;
+static int auth = 0;
+static int encrypt = 0;
+static int secure = 0;
+static int socktype = SOCK_SEQPACKET;
+static int linger = 0;
+static int reliable = 0;
 
-float tv2fl(struct timeval tv)
+static float tv2fl(struct timeval tv)
 {
 	return (float)tv.tv_sec + (float)(tv.tv_usec/1000000.0);
 }
 
-char *ltoh(unsigned long c, char* s)
+static char *ltoh(unsigned long c, char* s)
 {
 	int c1;
 
@@ -118,10 +113,10 @@ char *ltoh(unsigned long c, char* s)
 	c1     = c & 0x0f;
 	*(s++) = NIBBLE_TO_ASCII (c1);
 	*s     = 0;
-	return (s);
+	return s;
 }
 
-char *ctoh(char c, char* s)
+static char *ctoh(char c, char* s)
 {
 	char c1;
 
@@ -130,18 +125,18 @@ char *ctoh(char c, char* s)
 	c1     = c & 0x0f;
 	*(s++) = NIBBLE_TO_ASCII (c1);
 	*s     = 0;
-	return (s);
+	return s;
 }
 
-void hexdump(char *s, unsigned long l)
+static void hexdump(char *s, unsigned long l)
 {
 	char bfr[80];
 	char *pb;
 	unsigned long i, n = 0;
-	
+
 	if (l == 0)
 		return;
-	
+
 	while (n < l) {
 		pb = bfr;
 		pb = ltoh (n, pb);
@@ -162,36 +157,38 @@ void hexdump(char *s, unsigned long l)
 			else
 				*(pb++) = (isprint (*(s + i)) ? *(s + i) : '.');
 		}
-		*pb     = 0;
+		*pb = 0;
 		n += 16;
 		s += 16;
 		puts(bfr);
 	}
 }
 
-int do_connect(char *svr)
+static int do_connect(char *svr)
 {
 	struct sockaddr_l2 rem_addr, loc_addr;
 	struct l2cap_options opts;
+	struct l2cap_conninfo conn;
 	int s, opt;
 
 	if ((s = socket(PF_BLUETOOTH, socktype, BTPROTO_L2CAP)) < 0) {
-		syslog(LOG_ERR, "Can't create socket. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't create socket: %s (%d)", strerror(errno), errno);
 		return -1;
 	}
 
 	memset(&loc_addr, 0, sizeof(loc_addr));
 	loc_addr.l2_family = AF_BLUETOOTH;
-	loc_addr.l2_bdaddr = bdaddr;
+	bacpy(&loc_addr.l2_bdaddr, &bdaddr);
 	if (bind(s, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0) {
-		syslog(LOG_ERR, "Can't bind socket. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't bind socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	/* Get default options */
+	memset(&opts, 0, sizeof(opts));
 	opt = sizeof(opts);
 	if (getsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, &opt) < 0) {
-		syslog(LOG_ERR, "Can't get default L2CAP options. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't get default L2CAP options: %s (%d)", strerror(errno), errno);
 		return -1;	
 	}
 
@@ -199,7 +196,7 @@ int do_connect(char *svr)
 	opts.omtu = omtu;
 	opts.imtu = imtu;
 	if (setsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, opt) < 0) {
-		syslog(LOG_ERR, "Can't set L2CAP options. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set L2CAP options: %s (%d)", strerror(errno), errno);
 		return -1;
 	}
 
@@ -207,7 +204,7 @@ int do_connect(char *svr)
 	if (linger) {
 		struct linger l = { .l_onoff = 1, .l_linger = linger };
 		if (setsockopt(s, SOL_SOCKET, SO_LINGER, &l, sizeof(l)) < 0) {
-			syslog(LOG_ERR, "Can't enable SO_LINGER. %s(%d)",
+			syslog(LOG_ERR, "Can't enable SO_LINGER: %s (%d)",
 				strerror(errno), errno);
 			return -1;
 		}
@@ -216,53 +213,63 @@ int do_connect(char *svr)
 	/* Set link mode */
 	opt = 0;
 	if (reliable)
-		 opt |= L2CAP_LM_RELIABLE;
+		opt |= L2CAP_LM_RELIABLE;
 
 	if (setsockopt(s, SOL_L2CAP, L2CAP_LM, &opt, sizeof(opt)) < 0) {
-		syslog(LOG_ERR, "Can't set L2CAP link mode. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set L2CAP link mode: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	memset(&rem_addr, 0, sizeof(rem_addr));
 	rem_addr.l2_family = AF_BLUETOOTH;
-	baswap(&rem_addr.l2_bdaddr, strtoba(svr));
+	str2ba(svr, &rem_addr.l2_bdaddr);
 	rem_addr.l2_psm = htobs(psm);
-	if (connect(s, (struct sockaddr *)&rem_addr, sizeof(rem_addr)) < 0 ) {
-		syslog(LOG_ERR, "Can't connect. %s(%d)", strerror(errno), errno);
+	if (connect(s, (struct sockaddr *) &rem_addr, sizeof(rem_addr)) < 0 ) {
+		syslog(LOG_ERR, "Can't connect: %s (%d)", strerror(errno), errno);
 		close(s);
 		return -1;
 	}
 
+	memset(&opts, 0, sizeof(opts));
 	opt = sizeof(opts);
 	if (getsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, &opt) < 0) {
-		syslog(LOG_ERR, "Can't get L2CAP options. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't get L2CAP options: %s (%d)", strerror(errno), errno);
 		close(s);
 		return -1;
 	}
 
-	syslog(LOG_INFO, "Connected [imtu %d, omtu %d, flush_to %d]\n",
-	       opts.imtu, opts.omtu, opts.flush_to);
+	memset(&conn, 0, sizeof(conn));
+	opt = sizeof(conn);
+	if (getsockopt(s, SOL_L2CAP, L2CAP_CONNINFO, &conn, &opt) < 0) {
+		syslog(LOG_ERR, "Can't get L2CAP connection information: %s (%d)", strerror(errno), errno);
+		close(s);
+		return -1;
+	}
+
+	syslog(LOG_INFO, "Connected [imtu %d, omtu %d, flush_to %d, mode %d, handle %d, class 0x%02x%02x%02x]",
+		opts.imtu, opts.omtu, opts.flush_to, opts.mode, conn.hci_handle,
+		conn.dev_class[2], conn.dev_class[1], conn.dev_class[0]);
 
 	return s;
 }
 
-void do_listen(void (*handler)(int sk))
+static void do_listen(void (*handler)(int sk))
 {
 	struct sockaddr_l2 loc_addr, rem_addr;
 	struct l2cap_options opts;
-	int  s, s1, opt;
-	bdaddr_t ba;
+	int s, s1, opt;
+	char ba[18];
 
 	if ((s = socket(PF_BLUETOOTH, socktype, BTPROTO_L2CAP)) < 0) {
-		syslog(LOG_ERR, "Can't create socket. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't create socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	loc_addr.l2_family = AF_BLUETOOTH;
-	loc_addr.l2_bdaddr = bdaddr;
-	loc_addr.l2_psm    = htobs(psm);
+	bacpy(&loc_addr.l2_bdaddr, &bdaddr);
+	loc_addr.l2_psm = htobs(psm);
 	if (bind(s, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0) {
-		syslog(LOG_ERR, "Can't bind socket. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't bind socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
@@ -270,32 +277,31 @@ void do_listen(void (*handler)(int sk))
 	opt = 0;
 	if (reliable)
 		opt |= L2CAP_LM_RELIABLE;
-
 	if (master)
-		 opt |= L2CAP_LM_MASTER;
-
+		opt |= L2CAP_LM_MASTER;
 	if (auth)
-		 opt |= L2CAP_LM_AUTH;
-
+		opt |= L2CAP_LM_AUTH;
 	if (encrypt)
-		 opt |= L2CAP_LM_ENCRYPT;
+		opt |= L2CAP_LM_ENCRYPT;
+	if (secure)
+		opt |= L2CAP_LM_SECURE;
 
 	if (setsockopt(s, SOL_L2CAP, L2CAP_LM, &opt, sizeof(opt)) < 0) {
-		syslog(LOG_ERR, "Can't set L2CAP link mode. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set L2CAP link mode: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	/* Get default options */
 	opt = sizeof(opts);
 	if (getsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, &opt) < 0) {
-		syslog(LOG_ERR, "Can't get default L2CAP options. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't get default L2CAP options: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	/* Set new options */
 	opts.imtu = imtu;
 	if (setsockopt(s, SOL_L2CAP, L2CAP_OPTIONS, &opts, opt) < 0) {
-		syslog(LOG_ERR, "Can't set L2CAP options. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set L2CAP options: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
@@ -305,7 +311,7 @@ void do_listen(void (*handler)(int sk))
 	}
 
 	if (listen(s, 10)) {
-		syslog(LOG_ERR,"Can not listen on the socket. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR,"Can not listen on the socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
@@ -313,11 +319,11 @@ void do_listen(void (*handler)(int sk))
 
 	while(1) {
 		opt = sizeof(rem_addr);
-		if ((s1 = accept(s, (struct sockaddr *)&rem_addr, &opt)) < 0) {
-			syslog(LOG_ERR,"Accept failed. %s(%d)", strerror(errno), errno);
+		if ((s1 = accept(s, (struct sockaddr *) &rem_addr, &opt)) < 0) {
+			syslog(LOG_ERR,"Accept failed: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
-		if( fork() ) {
+		if (fork()) {
 			/* Parent */
 			close(s1);
 			continue;
@@ -328,19 +334,19 @@ void do_listen(void (*handler)(int sk))
 
 		opt = sizeof(opts);
 		if (getsockopt(s1, SOL_L2CAP, L2CAP_OPTIONS, &opts, &opt) < 0) {
-			syslog(LOG_ERR, "Can't get L2CAP options. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Can't get L2CAP options: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
 
-		baswap(&ba, &rem_addr.l2_bdaddr);
+		ba2str(&rem_addr.l2_bdaddr, ba);
 		syslog(LOG_INFO, "Connect from %s [imtu %d, omtu %d, flush_to %d]\n",
-		       batostr(&ba), opts.imtu, opts.omtu, opts.flush_to);
+					ba, opts.imtu, opts.omtu, opts.flush_to);
 
 		/* Enable SO_LINGER */
 		if (linger) {
 			struct linger l = { .l_onoff = 1, .l_linger = linger };
 			if (setsockopt(s, SOL_SOCKET, SO_LINGER, &l, sizeof(l)) < 0) {
-				syslog(LOG_ERR, "Can't enable SO_LINGER. %s(%d)",
+				syslog(LOG_ERR, "Can't enable SO_LINGER: %s (%d)",
 					strerror(errno), errno);
 				exit(1);
 			}
@@ -348,12 +354,12 @@ void do_listen(void (*handler)(int sk))
 
 		handler(s1);
 
-		syslog(LOG_INFO, "Disconnect. %m\n");
+		syslog(LOG_INFO, "Disconnect: %m");
 		exit(0);
 	}
 }
 
-void dump_mode(int s)
+static void dump_mode(int s)
 {
 	int len;
 	int opt, optl;
@@ -361,10 +367,10 @@ void dump_mode(int s)
 	syslog(LOG_INFO, "Receiving ...");
 	while (1) {
 		fd_set rset;
-	
+
 		FD_ZERO(&rset);
 		FD_SET(s, &rset);
-		
+
 		if (select(s + 1, &rset, NULL, NULL, NULL) < 0)
 			return;
 
@@ -378,8 +384,8 @@ void dump_mode(int s)
 					syslog(LOG_INFO, "L2CAP Error ECOMM - clearing error and continuing.\n");
 					optl = sizeof(opt);
 					if (getsockopt(s, SOL_SOCKET, SO_ERROR, &opt, &optl ) < 0) { // Clear error
-						syslog(LOG_ERR, "Couldn't getsockopt(SO_ERROR): %s(%d)\n",
-						       strerror(errno), errno);
+						syslog(LOG_ERR, "Couldn't getsockopt(SO_ERROR): %s (%d)\n",
+							strerror(errno), errno);
 						return;
 					}
 					continue;
@@ -395,7 +401,7 @@ void dump_mode(int s)
 	}
 }
 
-void recv_mode(int s)
+static void recv_mode(int s)
 {
 	struct timeval tv_beg,tv_end,tv_diff;
 	long total;
@@ -406,12 +412,12 @@ void recv_mode(int s)
 
 	seq = 0;
 	while (1) {
-		gettimeofday(&tv_beg,NULL);
+		gettimeofday(&tv_beg, NULL);
 		total = 0;
 		while (total < data_size) {
 			uint32_t sq;
 			uint16_t l;
-			int i,r;
+			int i, r;
 
 			if ((r = recv(s, buf, data_size, 0)) <= 0) {
 				if (r < 0) {
@@ -419,107 +425,106 @@ void recv_mode(int s)
 						syslog(LOG_INFO, "L2CAP Error ECOMM - clearing error and continuing.\n");
 						optl = sizeof(opt);
 						if (getsockopt(s, SOL_SOCKET, SO_ERROR, &opt, &optl ) < 0) { // Clear error
-							syslog(LOG_ERR, "Couldn't getsockopt(SO_ERROR): %s(%d)\n",
-							       strerror(errno), errno);
+							syslog(LOG_ERR, "Couldn't getsockopt(SO_ERROR): %s (%d)\n",
+								strerror(errno), errno);
 							return;
 						}
 						continue;
 					} else {
 						syslog(LOG_ERR, "Read failed. %s(%d)",
-						       strerror(errno), errno);
+							strerror(errno), errno);
 					}
 				}
 				return;
 			}
 
 			/* Check sequence */
-			sq = btohl(*(uint32_t *)buf);
+			sq = btohl(*(uint32_t *) buf);
 			if (seq != sq) {
 				syslog(LOG_INFO, "seq missmatch: %d -> %d", seq, sq);
 				seq = sq;
 			}
 			seq++;
-			
+
 			/* Check length */
-			l = btohs(*(uint16_t *)(buf+4));
+			l = btohs(*(uint16_t *) (buf + 4));
 			if (r != l) {
 				syslog(LOG_INFO, "size missmatch: %d -> %d", r, l);
 				continue;
 			}
-			
-			/* Verify data */	
-			for (i=6; i < r; i++) {
+
+			/* Verify data */
+			for (i = 6; i < r; i++) {
 				if (buf[i] != 0x7f)
 					syslog(LOG_INFO, "data missmatch: byte %d 0x%2.2x", i, buf[i]);
 			}
 
 			total += r;
 		}
-		gettimeofday(&tv_end,NULL);
+		gettimeofday(&tv_end, NULL);
 
-		timersub(&tv_end,&tv_beg,&tv_diff);
+		timersub(&tv_end, &tv_beg, &tv_diff);
 
-		syslog(LOG_INFO,"%ld bytes in %.2f sec, %.2f kB/s",total,
-		       tv2fl(tv_diff), (float)(total / tv2fl(tv_diff) ) / 1024.0);
+		syslog(LOG_INFO,"%ld bytes in %.2f sec, %.2f kB/s", total,
+			tv2fl(tv_diff), (float)(total / tv2fl(tv_diff) ) / 1024.0);
 	}
 }
 
-void send_mode(int s)
+static void send_mode(int s)
 {
 	uint32_t seq;
 	int i;
 
 	syslog(LOG_INFO, "Sending ...");
 
-	for(i = 6; i < data_size; i++)
+	for (i = 6; i < data_size; i++)
 		buf[i] = 0x7f;
 
 	seq = 0;
 	while ((num_frames == -1) || (num_frames-- > 0)) {
 		*(uint32_t *) buf = htobl(seq);
-		*(uint16_t *)(buf+4) = htobs(data_size);
+		*(uint16_t *) (buf + 4) = htobs(data_size);
 		seq++;
-		
+
 		if (send(s, buf, data_size, 0) <= 0) {
-			syslog(LOG_ERR, "Send failed. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Send failed: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
 	}
 
 	syslog(LOG_INFO, "Closing channel ...");
 	if (shutdown(s, SHUT_RDWR) < 0)
-		syslog(LOG_INFO, "Close failed. %m.");
+		syslog(LOG_INFO, "Close failed: %m");
 	else
 		syslog(LOG_INFO, "Done");
 }
 
-void senddump_mode(int s)
+static void senddump_mode(int s)
 {
 	uint32_t seq;
 	int i;
 
 	syslog(LOG_INFO, "Sending ...");
 
-	for(i = 6; i < data_size; i++)
+	for (i = 6; i < data_size; i++)
 		buf[i] = 0x7f;
 
 	seq = 0;
 	while ((num_frames == -1) || (num_frames-- > 0)) {
 		*(uint32_t *) buf = htobl(seq);
-		*(uint16_t *)(buf+4) = htobs(data_size);
+		*(uint16_t *) (buf + 4) = htobs(data_size);
 		seq++;
-		
+
 		if (send(s, buf, data_size, 0) <= 0) {
-			syslog(LOG_ERR, "Send failed. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Send failed: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
 	}
 
 	dump_mode(s);
-
 }
 
-void reconnect_mode(char *svr)
+static void reconnect_mode(char *svr)
 {
 	while(1) {
 		int s = do_connect(svr);
@@ -527,7 +532,7 @@ void reconnect_mode(char *svr)
 	}
 }
 
-void connect_mode(char *svr)
+static void connect_mode(char *svr)
 {
 	int s;
 	if ((s = do_connect(svr)) < 0)
@@ -535,7 +540,7 @@ void connect_mode(char *svr)
 	sleep(99999999);
 }
 
-void multi_connect_mode(char *svr)
+static void multi_connect_mode(char *svr)
 {
 	while (1) {
 		int i, s;
@@ -552,7 +557,7 @@ void multi_connect_mode(char *svr)
 	}
 }
 
-void usage(void)
+static void usage(void)
 {
 	printf("l2test - L2CAP testing\n"
 		"Usage:\n");
@@ -570,33 +575,33 @@ void usage(void)
 		"\t-m multiple connects\n");
 
 	printf("Options:\n"
-		"\t[-b bytes] [-S bdaddr] [-P psm]\n"
-	       	"\t[-I imtu] [-O omtu]\n"
+		"\t[-b bytes] [-i device] [-P psm]\n"
+		"\t[-I imtu] [-O omtu]\n"
 		"\t[-N num] send num frames (default = infinite)\n"
 		"\t[-L seconds] enable SO_LINGER\n"
 		"\t[-R] reliable mode\n"
 		"\t[-D] use connectionless channel (datagram)\n"
 		"\t[-A] request authentication\n"
 		"\t[-E] request encryption\n"
-	       	"\t[-M] become master\n");
+		"\t[-S] secure connection\n"
+		"\t[-M] become master\n");
 }
 
-extern int optind,opterr,optopt;
-extern char *optarg;
-
 int main(int argc ,char *argv[])
 {
 	int opt, mode, s, need_addr;
 	struct sigaction sa;
 
 	mode = RECV; need_addr = 0;
-	
-	while ((opt=getopt(argc,argv,"rdscuwmnxyb:P:I:O:S:N:RMAEDL:")) != EOF) {
+
+	bacpy(&bdaddr, BDADDR_ANY);
+
+	while ((opt=getopt(argc,argv,"rdscuwmnxyb:i:P:I:O:N:RMAESL:D")) != EOF) {
 		switch(opt) {
 		case 'r':
 			mode = RECV;
 			break;
-		
+
 		case 's':
 			mode = SEND;
 			need_addr = 1;
@@ -642,8 +647,11 @@ int main(int argc ,char *argv[])
 			mode = SENDDUMP;
 			break;
 
-		case 'S':
-			baswap(&bdaddr, strtoba(optarg));
+		case 'i':
+			if (!strncasecmp(optarg, "hci", 3))
+				hci_devba(atoi(optarg + 3), &bdaddr);
+			else
+				str2ba(optarg, &bdaddr);
 			break;
 
 		case 'P':
@@ -678,6 +686,10 @@ int main(int argc ,char *argv[])
 			encrypt = 1;
 			break;
 
+		case 'S':
+			secure = 1;
+			break;
+
 		case 'D':
 			socktype = SOCK_DGRAM;
 			break;
@@ -709,7 +721,7 @@ int main(int argc ,char *argv[])
 
 	openlog("l2test", LOG_PERROR | LOG_PID, LOG_LOCAL0);
 
-	switch( mode ){
+	switch (mode) {
 		case RECV:
 			do_listen(recv_mode);
 			break;
@@ -758,8 +770,8 @@ int main(int argc ,char *argv[])
 		case LSENDDUMP:
 			do_listen(senddump_mode);
 			break;
-
 	}
+
 	syslog(LOG_INFO, "Exit");
 
 	closelog();
