@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: rctest.c,v 1.5 2004/04/28 12:44:33 holtmann Exp $
+ *  $Id: rctest.c,v 1.12 2005/01/11 21:42:38 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,23 +32,18 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <sys/time.h>
-#include <unistd.h>
-#include <syslog.h>
-#include <string.h>
 #include <errno.h>
+#include <ctype.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <syslog.h>
 #include <signal.h>
-
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <resolv.h>
-#include <netdb.h>
+#include <sys/time.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
+#include <bluetooth/hci.h>
+#include <bluetooth/hci_lib.h>
 #include <bluetooth/rfcomm.h>
 
 /* Test modes */
@@ -63,38 +58,36 @@ enum {
 	LSEND
 };
 
-unsigned char *buf;
-
-/* Default mtu */
-int imtu = 672;
-int omtu = 0;
+static unsigned char *buf;
 
 /* Default data size */
-long data_size = 127;
-long num_frames = -1;
+static long data_size = 127;
+static long num_frames = -1;
 
 /* Default addr and channel */
-bdaddr_t bdaddr;
-uint8_t  channel = 10;
+static bdaddr_t bdaddr;
+static uint8_t channel = 10;
 
-int master = 0;
-int auth = 0;
-int encrypt = 0;
-int socktype = SOCK_STREAM;
-int linger = 0;
+static int master = 0;
+static int auth = 0;
+static int encrypt = 0;
+static int secure = 0;
+static int socktype = SOCK_STREAM;
+static int linger = 0;
 
-float tv2fl(struct timeval tv)
+static float tv2fl(struct timeval tv)
 {
 	return (float)tv.tv_sec + (float)(tv.tv_usec/1000000.0);
 }
 
-int do_connect(char *svr)
+static int do_connect(char *svr)
 {
 	struct sockaddr_rc rem_addr, loc_addr;
-	int s;
+	struct rfcomm_conninfo conn;
+	int s, opt;
 
-	if( (s = socket(PF_BLUETOOTH, socktype, BTPROTO_RFCOMM)) < 0 ) {
-		syslog(LOG_ERR, "Can't create socket. %s(%d)", strerror(errno), errno);
+	if ((s = socket(PF_BLUETOOTH, socktype, BTPROTO_RFCOMM)) < 0) {
+		syslog(LOG_ERR, "Can't create socket: %s (%d)", strerror(errno), errno);
 		return -1;
 	}
 
@@ -102,7 +95,7 @@ int do_connect(char *svr)
 	if (linger) {
 		struct linger l = { .l_onoff = 1, .l_linger = linger };
 		if (setsockopt(s, SOL_SOCKET, SO_LINGER, &l, sizeof(l)) < 0) {
-			syslog(LOG_ERR, "Can't enable SO_LINGER. %s(%d)",
+			syslog(LOG_ERR, "Can't enable SO_LINGER: %s (%d)",
 				strerror(errno), errno);
 			return -1;
 		}
@@ -110,66 +103,74 @@ int do_connect(char *svr)
 
 	memset(&loc_addr, 0, sizeof(loc_addr));
 	loc_addr.rc_family = AF_BLUETOOTH;
-	loc_addr.rc_bdaddr = bdaddr;
-	if( bind(s, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0 ) {
-		syslog(LOG_ERR, "Can't bind socket. %s(%d)", strerror(errno), errno);
+	bacpy(&loc_addr.rc_bdaddr, &bdaddr);
+	if (bind(s, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0) {
+		syslog(LOG_ERR, "Can't bind socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	memset(&rem_addr, 0, sizeof(rem_addr));
 	rem_addr.rc_family = AF_BLUETOOTH;
-	baswap(&rem_addr.rc_bdaddr, strtoba(svr));
+	str2ba(svr, &rem_addr.rc_bdaddr);
 	rem_addr.rc_channel = channel;
-	if( connect(s, (struct sockaddr *)&rem_addr, sizeof(rem_addr)) < 0 ){
-		syslog(LOG_ERR, "Can't connect. %s(%d)", strerror(errno), errno);
+	if (connect(s, (struct sockaddr *) &rem_addr, sizeof(rem_addr)) < 0) {
+		syslog(LOG_ERR, "Can't connect: %s (%d)", strerror(errno), errno);
 		close(s);
 		return -1;
 	}
 
-	syslog(LOG_INFO, "Connected");
+	memset(&conn, 0, sizeof(conn));
+	opt = sizeof(conn);
+	if (getsockopt(s, SOL_RFCOMM, RFCOMM_CONNINFO, &conn, &opt) < 0) {
+		syslog(LOG_ERR, "Can't get RFCOMM connection information: %s (%d)", strerror(errno), errno);
+		close(s);
+		//return -1;
+	}
+
+	syslog(LOG_INFO, "Connected [handle %d, class 0x%02x%02x%02x]",
+		conn.hci_handle,
+		conn.dev_class[2], conn.dev_class[1], conn.dev_class[0]);
 
 	return s;
 }
 
-void do_listen( void (*handler)(int sk) )
+static void do_listen(void (*handler)(int sk))
 {
 	struct sockaddr_rc loc_addr, rem_addr;
-	int  s, s1, opt;
-	bdaddr_t ba;
+	int s, s1, opt;
+	char ba[18];
 
-	if( (s = socket(PF_BLUETOOTH, socktype, BTPROTO_RFCOMM)) < 0 ) {
-		syslog(LOG_ERR, "Can't create socket. %s(%d)", strerror(errno), errno);
+	if ((s = socket(PF_BLUETOOTH, socktype, BTPROTO_RFCOMM)) < 0) {
+		syslog(LOG_ERR, "Can't create socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
 	loc_addr.rc_family = AF_BLUETOOTH;
-	loc_addr.rc_bdaddr = bdaddr;
+	bacpy(&loc_addr.rc_bdaddr, &bdaddr);
 	loc_addr.rc_channel = channel;
-	if( bind(s, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0 ) {
-		syslog(LOG_ERR, "Can't bind socket. %s(%d)", strerror(errno), errno);
+	if (bind(s, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0) {
+		syslog(LOG_ERR, "Can't bind socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
-#if 0
 	/* Set link mode */
 	opt = 0;
 	if (master)
-		 opt |= L2CAP_LM_MASTER;
-
+		opt |= RFCOMM_LM_MASTER;
 	if (auth)
-		 opt |= L2CAP_LM_AUTH;
-
+		opt |= RFCOMM_LM_AUTH;
 	if (encrypt)
-		 opt |= L2CAP_LM_ENCRYPT;
+		opt |= RFCOMM_LM_ENCRYPT;
+	if (secure)
+		opt |= RFCOMM_LM_SECURE;
 
-	if (setsockopt(s, SOL_RFCOMM, L2CAP_LM, &opt, sizeof(opt)) < 0) {
-		syslog(LOG_ERR, "Can't set L2CAP link mode. %s(%d)", strerror(errno), errno);
+	if (setsockopt(s, SOL_RFCOMM, RFCOMM_LM, &opt, sizeof(opt)) < 0) {
+		syslog(LOG_ERR, "Can't set RFCOMM link mode: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
-#endif
 
-	if( listen(s, 10) ) {
-		syslog(LOG_ERR,"Can not listen on the socket. %s(%d)", strerror(errno), errno);
+	if (listen(s, 10)) {
+		syslog(LOG_ERR,"Can not listen on the socket: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
@@ -177,11 +178,11 @@ void do_listen( void (*handler)(int sk) )
 
 	while(1) {
 		opt = sizeof(rem_addr);
-		if( (s1 = accept(s, (struct sockaddr *)&rem_addr, &opt)) < 0 ) {
-			syslog(LOG_ERR,"Accept failed. %s(%d)", strerror(errno), errno);
+		if ((s1 = accept(s, (struct sockaddr *) &rem_addr, &opt)) < 0) {
+			syslog(LOG_ERR,"Accept failed: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
-		if( fork() ) {
+		if (fork()) {
 			/* Parent */
 			close(s1);
 			continue;
@@ -190,14 +191,14 @@ void do_listen( void (*handler)(int sk) )
 
 		close(s);
 
-		baswap(&ba, &rem_addr.rc_bdaddr);
-		syslog(LOG_INFO, "Connect from %s \n", batostr(&ba));
+		ba2str(&rem_addr.rc_bdaddr, ba);
+		syslog(LOG_INFO, "Connect from %s", ba);
 
 		/* Enable SO_LINGER */
 		if (linger) {
 			struct linger l = { .l_onoff = 1, .l_linger = linger };
 			if (setsockopt(s, SOL_SOCKET, SO_LINGER, &l, sizeof(l)) < 0) {
-				syslog(LOG_ERR, "Can't enable SO_LINGER. %s(%d)",
+				syslog(LOG_ERR, "Can't enable SO_LINGER: %s (%d)",
 					strerror(errno), errno);
 				exit(1);
 			}
@@ -205,21 +206,21 @@ void do_listen( void (*handler)(int sk) )
 
 		handler(s1);
 
-		syslog(LOG_INFO, "Disconnect\n");
+		syslog(LOG_INFO, "Disconnect: %m");
 		exit(0);
 	}
 }
 
-void dump_mode(int s)
+static void dump_mode(int s)
 {
 	int len;
 
 	syslog(LOG_INFO, "Receiving ...");
 	while ((len = read(s, buf, data_size)) > 0)
-		syslog(LOG_INFO, "Recevied %d bytes\n", len);
+		syslog(LOG_INFO, "Recevied %d bytes", len);
 }
 
-void recv_mode(int s)
+static void recv_mode(int s)
 {
 	struct timeval tv_beg,tv_end,tv_diff;
 	long total;
@@ -238,13 +239,13 @@ void recv_mode(int s)
 
 			if ((r = recv(s, buf, data_size, 0)) <= 0) {
 				if (r < 0)
-					syslog(LOG_ERR, "Read failed. %s(%d)",
+					syslog(LOG_ERR, "Read failed: %s (%d)",
 							strerror(errno), errno);
 				return;	
 			}
 #if 0
 			/* Check sequence */
-			sq = btohl(*(uint32_t *)buf);
+			sq = btohl(*(uint32_t *) buf);
 			if (seq != sq) {
 				syslog(LOG_INFO, "seq missmatch: %d -> %d", seq, sq);
 				seq = sq;
@@ -252,14 +253,14 @@ void recv_mode(int s)
 			seq++;
 			
 			/* Check length */
-			l = btohs(*(uint16_t *)(buf+4));
+			l = btohs(*(uint16_t *) (buf + 4));
 			if (r != l) {
 				syslog(LOG_INFO, "size missmatch: %d -> %d", r, l);
 				continue;
 			}
 			
 			/* Verify data */	
-			for (i=6; i < r; i++) {
+			for (i = 6; i < r; i++) {
 				if (buf[i] != 0x7f)
 					syslog(LOG_INFO, "data missmatch: byte %d 0x%2.2x", i, buf[i]);
 			}
@@ -271,40 +272,40 @@ void recv_mode(int s)
 		timersub(&tv_end,&tv_beg,&tv_diff);
 
 		syslog(LOG_INFO,"%ld bytes in %.2f sec, %.2f kB/s",total,
-		       tv2fl(tv_diff), (float)(total / tv2fl(tv_diff) ) / 1024.0);
+			tv2fl(tv_diff), (float)(total / tv2fl(tv_diff) ) / 1024.0);
 	}
 }
 
-void send_mode(int s)
+static void send_mode(int s)
 {
 	uint32_t seq;
 	int i;
 
 	syslog(LOG_INFO,"Sending ...");
 
-	for(i=6; i < data_size; i++)
-		buf[i]=0x7f;
+	for (i = 6; i < data_size; i++)
+		buf[i] = 0x7f;
 
 	seq = 0;
 	while ((num_frames == -1) || (num_frames-- > 0)) {
 		*(uint32_t *) buf = htobl(seq);
-		*(uint16_t *)(buf+4) = htobs(data_size);
+		*(uint16_t *) (buf + 4) = htobs(data_size);
 		seq++;
 		
 		if (send(s, buf, data_size, 0) <= 0) {
-			syslog(LOG_ERR, "Send failed. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Send failed: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
 	}
 
 	syslog(LOG_INFO, "Closing channel ...");
 	if (shutdown(s, SHUT_RDWR) < 0)
-		syslog(LOG_INFO, "Close failed. %m.");
+		syslog(LOG_INFO, "Close failed: %m");
 	else
 		syslog(LOG_INFO, "Done");
 }
 
-void reconnect_mode(char *svr)
+static void reconnect_mode(char *svr)
 {
 	while(1) {
 		int s = do_connect(svr);
@@ -312,11 +313,11 @@ void reconnect_mode(char *svr)
 	}
 }
 
-void multi_connect_mode(char *svr)
+static void multi_connect_mode(char *svr)
 {
 	while (1) {
 		int i, s;
-		for (i=0; i<10; i++) {
+		for (i = 0; i < 10; i++) {
 			if (fork()) continue;
 
 			/* Child */
@@ -329,7 +330,7 @@ void multi_connect_mode(char *svr)
 	}
 }
 
-void usage(void)
+static void usage(void)
 {
 	printf("rctest - RFCOMM testing\n"
 		"Usage:\n");
@@ -345,31 +346,29 @@ void usage(void)
 		"\t-m multiple connects\n");
 
 	printf("Options:\n"
-		"\t[-b bytes] [-S bdaddr] [-P channel]\n"
-	       	"\t[-I imtu] [-O omtu]\n"
-	       	"\t[-L seconds] enabled SO_LINGER option\n"
-	       	"\t[-N num] number of frames to send\n"
+		"\t[-b bytes] [-i device] [-P channel]\n"
+		"\t[-L seconds] enabled SO_LINGER option\n"
+		"\t[-N num] number of frames to send\n"
+		"\t[-A] request authentication\n"
 		"\t[-E] request encryption\n"
-		"\t[-E] request encryption\n"
-	       	"\t[-M] become master\n");
+		"\t[-M] become master\n");
 }
 
-extern int optind,opterr,optopt;
-extern char *optarg;
-
 int main(int argc ,char *argv[])
 {
 	int opt, mode, s, need_addr;
 	struct sigaction sa;
 
 	mode = RECV; need_addr = 0;
-	
-	while ((opt=getopt(argc,argv,"rdscuwmnb:P:I:O:S:MAEL:N:")) != EOF) {
-		switch(opt) {
+
+	bacpy(&bdaddr, BDADDR_ANY);
+
+	while ((opt=getopt(argc,argv,"rdscuwmnb:i:P:N:MAESL:")) != EOF) {
+		switch (opt) {
 		case 'r':
 			mode = RECV;
 			break;
-		
+
 		case 's':
 			mode = SEND;
 			need_addr = 1;
@@ -407,22 +406,17 @@ int main(int argc ,char *argv[])
 			data_size = atoi(optarg);
 			break;
 
-		case 'S':
-			baswap(&bdaddr, strtoba(optarg));
+		case 'i':
+			if (!strncasecmp(optarg, "hci", 3))
+				hci_devba(atoi(optarg + 3), &bdaddr);
+			else
+				str2ba(optarg, &bdaddr);
 			break;
 
 		case 'P':
 			channel = atoi(optarg);
 			break;
 
-		case 'I':
-			imtu = atoi(optarg);
-			break;
-
-		case 'O':
-			omtu = atoi(optarg);
-			break;
-
 		case 'M':
 			master = 1;
 			break;
@@ -435,6 +429,10 @@ int main(int argc ,char *argv[])
 			encrypt = 1;
 			break;
 
+		case 'S':
+			secure = 1;
+			break;
+
 		case 'L':
 			linger = atoi(optarg);
 			break;
@@ -466,7 +464,7 @@ int main(int argc ,char *argv[])
 
 	openlog("rctest", LOG_PERROR | LOG_PID, LOG_LOCAL0);
 
-	switch( mode ){
+	switch (mode) {
 		case RECV:
 			do_listen(recv_mode);
 			break;
@@ -508,6 +506,7 @@ int main(int argc ,char *argv[])
 			dump_mode(s);
 			break;
 	}
+
 	syslog(LOG_INFO, "Exit");
 
 	closelog();
