@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: csr.c,v 1.3 2004/06/03 09:59:40 holtmann Exp $
+ *  $Id: csr.c,v 1.10 2004/11/29 18:55:53 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,9 @@ static struct {
 	uint16_t id;
 	char *str;
 } csr_map[] = {
+	{   66, "HCI 9.8"	},
+	{   97, "HCI 10.3"	},
+	{  101, "HCI 10.5"	},
 	{  111,	"HCI 11.0"	},
 	{  112,	"HCI 11.1"	},
 	{  114,	"HCI 11.2"	},
@@ -242,10 +245,27 @@ static struct {
 	{  856,	"HCI 17.3.2"	},
 	{  857,	"HCI 17.3.2"	},
 	{  858,	"HCI 17.3.2"	},
+	{ 1120, "HCI 17.11"	},
 	{ 1168, "HCI 18.1"	},
 	{ 1169, "HCI 18.1"	},
+	{ 1241, "HCI 18.x"	},
+	{ 1298, "HCI 18.2"	},
+	{ 1354, "HCI 18.2"	},
 	{ 1392, "HCI 18.2"	},
 	{ 1393, "HCI 18.2"	},
+	{ 1501, "HCI 18.2"	},
+	{ 1503, "HCI 18.2"	},
+	{ 1504, "HCI 18.2"	},
+	{ 1505, "HCI 18.2"	},
+	{ 1506, "HCI 18.2"	},
+	{ 1520, "HCI 18.2"	},
+	{ 1586, "HCI 18.2"	},
+	{ 1591, "HCI 18.2"	},
+	{ 1592, "HCI 18.2"	},
+	{ 1733, "HCI 18.3"	},
+	{ 1734, "HCI 18.3"	},
+	{ 1735, "HCI 18.3"	},
+	{ 1737, "HCI 18.3"	},
 	{    0, }
 };
 
@@ -273,10 +293,31 @@ char *csr_chipvertostr(uint16_t ver, uint16_t rev)
 		else
 			return "BlueCore01b";
 	case 0x02:
-		if (rev == 0x89)
-			return "BlueCore02 (ES2)";
-		else
+		switch (rev) {
+		case 0x89:
+			return "BlueCore02-External (ES2)";
+		case 0x8a:
+			return "BlueCore02-External";
+		case 0x28:
+			return "BlueCore02-ROM/Audio/Flash";
+		default:
 			return "BlueCore02";
+		}
+	case 0x03:
+		switch (rev) {
+		case 0x43:
+			return "BlueCore3-MM";
+		case 0x15:
+			return "BlueCore3-ROM";
+		case 0xe2:
+			return "BlueCore3-Flash";
+		case 0x26:
+			return "BlueCore4-External";
+		case 0x30:
+			return "BlueCore4-ROM";
+		default:
+			return "BlueCore3 or BlueCore4";
+		}
 	default:
 		return "Unknown";
 	}
