@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hciattach.c,v 1.23 2004/06/23 16:30:41 holtmann Exp $
+ *  $Id: hciattach.c,v 1.28 2005/01/11 18:46:46 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,21 +33,18 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 #include <signal.h>
-#include <fcntl.h>
 #include <syslog.h>
-#include <errno.h>
-#include <time.h>
 #include <termios.h>
-#include <fcntl.h>
-
+#include <time.h>
+#include <sys/time.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <sys/time.h>
-#include <asm/types.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
@@ -84,12 +81,18 @@ static int uart_speed(int s)
 		return B230400;
 	case 460800:
 		return B460800;
+	case 500000:
+		return B500000;
+	case 576000:
+		return B576000;
 	case 921600:
 		return B921600;
 	case 1000000:
 		return B1000000;
 	case 1152000:
 		return B1152000;
+	case 1500000:
+		return B1500000;
 	default:
 		return B57600;
 	}
@@ -836,6 +839,11 @@ struct uart_t uart[] = {
 	/* AmbiCom BT2000C Bluetooth PC/CF Card */
 	{ "bt2000c",  0x022d, 0x2000, HCI_UART_H4,    57600, 460800, FLOW_CTL, csr  },
 
+	/* Zoom Bluetooth PCMCIA Card */
+	{ "zoom",     0x0279, 0x950b, HCI_UART_BCSP, 115200, 115200, 0,        bcsp  },
+
+	/* Sitecom CN-504 PCMCIA Card */
+	{ "sitecom",  0x0279, 0x950b, HCI_UART_BCSP, 115200, 115200, 0,        bcsp  },
 	{ NULL, 0 }
 };
 
@@ -899,8 +907,10 @@ int init_uart(char *dev, struct uart_t *u, int send_break)
 
 	tcflush(fd, TCIOFLUSH);
 
-	if (send_break)
+	if (send_break) {
 		tcsendbreak(fd, 0);
+		usleep(500000);
+	}
 
 	if (u->init && u->init(fd, u, &ti) < 0)
 		return -1;
@@ -932,7 +942,7 @@ static void usage(void)
 {
 	printf("hciattach - HCI UART driver initialization utility\n");
 	printf("Usage:\n");
-	printf("\thciattach [-n] [-p] [-b] [-t timeout] [-s initial_speed] <tty> <type | id> [speed] [flow]\n");
+	printf("\thciattach [-n] [-p] [-b] [-t timeout] [-s initial_speed] <tty> <type | id> [speed] [flow|noflow]\n");
 	printf("\thciattach -l\n");
 }
 
