@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hciconfig.c,v 1.43 2004/07/27 16:43:40 holtmann Exp $
+ *  $Id: hciconfig.c,v 1.51 2005/01/11 13:33:14 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,17 +33,11 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <signal.h>
-#include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <getopt.h>
-
-#include <termios.h>
-#include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
@@ -53,56 +47,54 @@
 
 #include "csr.h"
 
-extern int optind,opterr,optopt;
-extern char *optarg;
-
 static struct hci_dev_info di;
 static int all;
 
-void print_dev_hdr(struct hci_dev_info *di);
-void print_dev_info(int ctl, struct hci_dev_info *di);
+static void print_dev_hdr(struct hci_dev_info *di);
+static void print_dev_info(int ctl, struct hci_dev_info *di);
 
-void print_dev_list(int ctl, int flags)
+static void print_dev_list(int ctl, int flags)
 {
 	struct hci_dev_list_req *dl;
 	struct hci_dev_req *dr;
 	int i;
 
-	if( !(dl = malloc(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t))) ) {
+	if (!(dl = malloc(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t)))) {
 		perror("Can't allocate memory");
 		exit(1);
 	}
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
-	if( ioctl(ctl, HCIGETDEVLIST, (void*)dl) ) {
+	if (ioctl(ctl, HCIGETDEVLIST, (void *) dl) < 0) {
 		perror("Can't get device list");
 		exit(1);
 	}
-	for(i=0; i< dl->dev_num; i++) {
+
+	for (i = 0; i< dl->dev_num; i++) {
 		di.dev_id = (dr+i)->dev_id;
-		if( ioctl(ctl, HCIGETDEVINFO, (void*)&di) )
+		if (ioctl(ctl, HCIGETDEVINFO, (void *) &di) < 0)
 			continue;
 		print_dev_info(ctl, &di);
 	}
 }
 
-void print_pkt_type(struct hci_dev_info *di)
+static void print_pkt_type(struct hci_dev_info *di)
 {
 	printf("\tPacket type: %s\n", hci_ptypetostr(di->pkt_type));
 }
 
-void print_link_policy(struct hci_dev_info *di)
+static void print_link_policy(struct hci_dev_info *di)
 {
 	printf("\tLink policy: %s\n", hci_lptostr(di->link_policy));
 }
 
-void print_link_mode(struct hci_dev_info *di)
+static void print_link_mode(struct hci_dev_info *di)
 {
 	printf("\tLink mode: %s\n", hci_lmtostr(di->link_mode));
 }
 
-void print_dev_features(struct hci_dev_info *di, int format)
+static void print_dev_features(struct hci_dev_info *di, int format)
 {
 	if (!format) {
 		printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n", 
@@ -120,41 +112,43 @@ void print_dev_features(struct hci_dev_info *di, int format)
 	}
 }
 
-void cmd_rstat(int ctl, int hdev, char *opt)
+static void cmd_rstat(int ctl, int hdev, char *opt)
 {
 	/* Reset HCI device stat counters */
-	if( ioctl(ctl, HCIDEVRESTAT, hdev) < 0 ) {
-		printf("Can't reset stats counters hci%d. %s(%d)\n", hdev, 
-				strerror(errno), errno);
+	if (ioctl(ctl, HCIDEVRESTAT, hdev) < 0) {
+		fprintf(stderr, "Can't reset stats counters hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_scan(int ctl, int hdev, char *opt)
+static void cmd_scan(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr;
 
 	dr.dev_id  = hdev;
 	dr.dev_opt = SCAN_DISABLED;
-	if( !strcmp(opt, "iscan") )
+	if (!strcmp(opt, "iscan"))
 		dr.dev_opt = SCAN_INQUIRY;
-	else if( !strcmp(opt, "pscan") )
+	else if (!strcmp(opt, "pscan"))
 		dr.dev_opt = SCAN_PAGE;
-	else if( !strcmp(opt, "piscan") )
+	else if (!strcmp(opt, "piscan"))
 		dr.dev_opt = SCAN_PAGE | SCAN_INQUIRY;
 
-	if( ioctl(ctl, HCISETSCAN, (unsigned long)&dr) < 0 ) {
-		printf("Can't set scan mode on hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+	if (ioctl(ctl, HCISETSCAN, (unsigned long) &dr) < 0) {
+		fprintf(stderr, "Can't set scan mode on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_iac(int ctl, int hdev, char *opt)
+static void cmd_iac(int ctl, int hdev, char *opt)
 {
 	int s = hci_open_dev(hdev);
 
 	if (s < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 	if (opt) {
@@ -193,82 +187,104 @@ void cmd_iac(int ctl, int hdev, char *opt)
 	close(s);
 }
 
-void cmd_auth(int ctl, int hdev, char *opt)
+static void cmd_auth(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr;
 
 	dr.dev_id = hdev;
-	if( !strcmp(opt, "auth") )
+	if (!strcmp(opt, "auth"))
 		dr.dev_opt = AUTH_ENABLED;
 	else
 		dr.dev_opt = AUTH_DISABLED;
 
-	if( ioctl(ctl, HCISETAUTH, (unsigned long)&dr) < 0 ) {
-		printf("Can't set auth on hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+	if (ioctl(ctl, HCISETAUTH, (unsigned long) &dr) < 0) {
+		fprintf(stderr, "Can't set auth on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_encrypt(int ctl, int hdev, char *opt)
+static void cmd_encrypt(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr;
 
 	dr.dev_id = hdev;
-	if( !strcmp(opt, "encrypt") )
+	if (!strcmp(opt, "encrypt"))
 		dr.dev_opt = ENCRYPT_P2P;
 	else
 		dr.dev_opt = ENCRYPT_DISABLED;
 
-	if( ioctl(ctl, HCISETENCRYPT, (unsigned long)&dr) < 0 ) {
-		printf("Can't set encrypt on hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+	if (ioctl(ctl, HCISETENCRYPT, (unsigned long) &dr) < 0) {
+		fprintf(stderr, "Can't set encrypt on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_up(int ctl, int hdev, char *opt)
+static void cmd_secmgr(int ctl, int hdev, char *opt)
 {
-	int ret;
+	int val, s = hci_open_dev(hdev);
 
-	/* Start HCI device */
-	if( (ret = ioctl(ctl, HCIDEVUP, hdev)) < 0 ) {
-		if( errno == EALREADY )
-			return;
-		printf("Can't init device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+	if (!strcmp(opt, "secmgr"))
+		val = 1;
+	else
+		val = 0;
+
+	if (ioctl(s, HCISETSECMGR, val) < 0) {
+		fprintf(stderr, "Can't set security manager on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
-	cmd_scan(ctl,  hdev, "piscan");
+	close(s);
 }
 
-void cmd_down(int ctl, int hdev, char *opt)
+static void cmd_up(int ctl, int hdev, char *opt)
+{
+	/* Start HCI device */
+	if (ioctl(ctl, HCIDEVUP, hdev) < 0) {
+		if (errno == EALREADY)
+			return;
+		fprintf(stderr, "Can't init device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+	cmd_scan(ctl, hdev, "piscan");
+}
+
+static void cmd_down(int ctl, int hdev, char *opt)
 {
 	/* Stop HCI device */
 	if (ioctl(ctl, HCIDEVDOWN, hdev) < 0) {
-		printf("Can't down device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't down device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_reset(int ctl, int hdev, char *opt)
+static void cmd_reset(int ctl, int hdev, char *opt)
 {
-	/* Reset HCI device
-	if( ioctl(ctl, HCIDEVRESET, hdev) < 0 ){
-	   printf("Reset failed hci%d. %s(%d)\n", hdev, strerror(errno), errno);
-	   exit(1);
+	/* Reset HCI device */
+#if 0
+	if (ioctl(ctl, HCIDEVRESET, hdev) < 0 ){
+		fprintf(stderr, "Reset failed for device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
 	}
-	*/
+#endif
 	cmd_down(ctl, hdev, "down");
 	cmd_up(ctl, hdev, "up");
 }
 
-void cmd_ptype(int ctl, int hdev, char *opt)
+static void cmd_ptype(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr;
 
 	dr.dev_id = hdev;
 
 	if (hci_strtoptype(opt, &dr.dev_opt)) {
-		if (ioctl(ctl, HCISETPTYPE, (unsigned long)&dr) < 0) {
-			printf("Can't set pkttype on hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		if (ioctl(ctl, HCISETPTYPE, (unsigned long) &dr) < 0) {
+			fprintf(stderr, "Can't set pkttype on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
@@ -277,16 +293,16 @@ void cmd_ptype(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_lp(int ctl, int hdev, char *opt)
+static void cmd_lp(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr;
 
 	dr.dev_id = hdev;
 
 	if (hci_strtolp(opt, &dr.dev_opt)) {
-		if (ioctl(ctl, HCISETLINKPOL, (unsigned long)&dr) < 0) {
-			printf("Can't set link policy on hci%d. %s(%d)\n", 
-					hdev, strerror(errno), errno);
+		if (ioctl(ctl, HCISETLINKPOL, (unsigned long) &dr) < 0) {
+			fprintf(stderr, "Can't set link policy on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
@@ -295,16 +311,16 @@ void cmd_lp(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_lm(int ctl, int hdev, char *opt)
+static void cmd_lm(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr;
 
 	dr.dev_id = hdev;
 
 	if (hci_strtolm(opt, &dr.dev_opt)) {
-		if (ioctl(ctl, HCISETLINKMODE, (unsigned long)&dr) < 0) {
-			printf("Can't set default link mode on hci%d. %s(%d)\n", 
-					hdev, strerror(errno), errno);
+		if (ioctl(ctl, HCISETLINKMODE, (unsigned long) &dr) < 0) {
+			fprintf(stderr, "Can't set default link mode on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
@@ -313,7 +329,7 @@ void cmd_lm(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_aclmtu(int ctl, int hdev, char *opt)
+static void cmd_aclmtu(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr = { dev_id: hdev };
 	uint16_t mtu, mpkt;
@@ -327,14 +343,14 @@ void cmd_aclmtu(int ctl, int hdev, char *opt)
 	*((uint16_t *)&dr.dev_opt + 1) = mtu;
 	*((uint16_t *)&dr.dev_opt + 0) = mpkt;
 
-	if (ioctl(ctl, HCISETACLMTU, (unsigned long)&dr) < 0) {
-		printf("Can't set ACL mtu on hci%d. %s(%d)\n", 
-				hdev, strerror(errno), errno);
+	if (ioctl(ctl, HCISETACLMTU, (unsigned long) &dr) < 0) {
+		fprintf(stderr, "Can't set ACL mtu on hci%d: %s(%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_scomtu(int ctl, int hdev, char *opt)
+static void cmd_scomtu(int ctl, int hdev, char *opt)
 {
 	struct hci_dev_req dr = { dev_id: hdev };
 	uint16_t mtu, mpkt;
@@ -348,39 +364,40 @@ void cmd_scomtu(int ctl, int hdev, char *opt)
 	*((uint16_t *)&dr.dev_opt + 1) = mtu;
 	*((uint16_t *)&dr.dev_opt + 0) = mpkt;
 	
-	if (ioctl(ctl, HCISETSCOMTU, (unsigned long)&dr) < 0) {
-		printf("Can't set SCO mtu on hci%d. %s(%d)\n", 
-				hdev, strerror(errno), errno);
+	if (ioctl(ctl, HCISETSCOMTU, (unsigned long) &dr) < 0) {
+		fprintf(stderr, "Can't set SCO mtu on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 }
 
-void cmd_features(int ctl, int hdev, char *opt)
+static void cmd_features(int ctl, int hdev, char *opt)
 {
 	print_dev_hdr(&di);
 	print_dev_features(&di, 1);
 }
 
-void cmd_name(int ctl, int hdev, char *opt)
+static void cmd_name(int ctl, int hdev, char *opt)
 {
 	int s = hci_open_dev(hdev);
 
 	if (s < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 	if (opt) {
-		if (hci_write_local_name(s, opt, 1000) < 0) {
-			printf("Can't change local name on hci%d. %s(%d)\n", 
-				hdev, strerror(errno), errno);
+		if (hci_write_local_name(s, opt, 2000) < 0) {
+			fprintf(stderr, "Can't change local name on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
 		char name[248];
 		int i;
 		if (hci_read_local_name(s, sizeof(name), name, 1000) < 0) {
-			printf("Can't read local name on hci%d. %s(%d)\n", 
-				hdev, strerror(errno), errno);
+			fprintf(stderr, "Can't read local name on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 		for (i = 0; i < 248 && name[i]; i++)
@@ -397,10 +414,10 @@ void cmd_name(int ctl, int hdev, char *opt)
  */
 static char *get_minor_device_name(int major, int minor)
 {
-	switch(major) {
-	case 0:/* misc */
+	switch (major) {
+	case 0:	/* misc */
 		return "";
-	case 1:/* computer */
+	case 1:	/* computer */
 		switch(minor) {
 		case 0:
 			return "Uncategorized";
@@ -418,7 +435,7 @@ static char *get_minor_device_name(int major, int minor)
 			return "Wearable";
 		}
 		break;
-	case 2:/* phone */
+	case 2:	/* phone */
 		switch(minor) {
 		case 0:
 			return "Uncategorized";
@@ -436,7 +453,7 @@ static char *get_minor_device_name(int major, int minor)
 			return "Sim Card Reader";
 		}
 		break;
-	case 3:/* lan access */
+	case 3:	/* lan access */
 		if (minor == 0)
 			return "Uncategorized";
 		switch(minor / 8) {
@@ -458,7 +475,7 @@ static char *get_minor_device_name(int major, int minor)
 			return "No service available";
 		}
 		break;
-	case 4:/* audio/video */
+	case 4:	/* audio/video */
 		switch(minor) {
 		case 0:
 			return "Uncategorized";
@@ -498,7 +515,7 @@ static char *get_minor_device_name(int major, int minor)
 			return "Gaming/Toy";
 		}
 		break;
-	case 5:/* peripheral */
+	case 5:	/* peripheral */
 		switch(minor) {
 		case 16:
 			return "Keyboard";
@@ -508,7 +525,7 @@ static char *get_minor_device_name(int major, int minor)
 			return "Combo keyboard/pointing device";
 		}
 		break;
-	case 6:/* imaging */
+	case 6:	/* imaging */
 		if (minor & 4)
 			return "Display";
 		if (minor & 8)
@@ -518,13 +535,13 @@ static char *get_minor_device_name(int major, int minor)
 		if (minor & 32)
 			return "Printer";
 		break;
-	case 63:/* uncategorised */
+	case 63:	/* uncategorised */
 		return "";
 	}
 	return "Unknown (reserved) minor device class";
 }
 
-void cmd_class(int ctl, int hdev, char *opt)
+static void cmd_class(int ctl, int hdev, char *opt)
 {
 	static char *services[] = { "Positioning",
 					"Networking",
@@ -545,21 +562,22 @@ void cmd_class(int ctl, int hdev, char *opt)
 	int s = hci_open_dev(hdev);
 
 	if (s < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 	if (opt) {
 		uint32_t cod = strtoul(opt, NULL, 16);
-		if (0 > hci_write_class_of_dev(s, cod, 1000)) {
-			printf("Can't write local class of device on hci%d. %s(%d)\n", 
-				hdev, strerror(errno), errno);
+		if (hci_write_class_of_dev(s, cod, 2000) < 0) {
+			fprintf(stderr, "Can't write local class of device on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
 		uint8_t cls[3];
-		if (0 > hci_read_class_of_dev(s, cls, 1000)) {
-			printf("Can't read class of device on hci%d. %s(%d)\n", 
-				hdev, strerror(errno), errno);
+		if (hci_read_class_of_dev(s, cls, 1000) < 0) {
+			fprintf(stderr, "Can't read class of device on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 		print_dev_hdr(&di);
@@ -567,7 +585,7 @@ void cmd_class(int ctl, int hdev, char *opt)
 		printf("\tService Classes: ");
 		if (cls[2]) {
 			int first = 1;
-			for(s=0; s < sizeof(services); s++)
+			for (s = 0; s < sizeof(*services); s++)
 				if (cls[2] & (1 << s)) {
 					if (!first)
 						printf(", ");
@@ -577,7 +595,7 @@ void cmd_class(int ctl, int hdev, char *opt)
 		} else
 			printf("Unspecified");
 		printf("\n\tDevice Class: ");
-		if ((cls[1] & 0x1f) > sizeof(major_devices))
+		if ((cls[1] & 0x1f) > sizeof(*major_devices))
 			printf("Invalid Device Class!\n");
 		else
 			printf("%s, %s\n", major_devices[cls[1] & 0x1f], 
@@ -585,7 +603,7 @@ void cmd_class(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_voice(int ctl, int hdev, char *opt)
+static void cmd_voice(int ctl, int hdev, char *opt)
 {
 	static char *icf[] = { "Linear", "u-Law", "A-Law", "Reserved" };
 	static char *idf[] = { "1's complement", "2's complement", "Sign-Magnitude", "Reserved" };
@@ -594,22 +612,23 @@ void cmd_voice(int ctl, int hdev, char *opt)
 	int s = hci_open_dev(hdev);
 
 	if (s < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 	if (opt) {
 		uint16_t vs = htobs(strtoul(opt, NULL, 16));
-		if (0 > hci_write_voice_setting(s, vs, 1000)) {
-			printf("Can't write voice setting on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+		if (hci_write_voice_setting(s, vs, 2000) < 0) {
+			fprintf(stderr, "Can't write voice setting on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
 		uint16_t vs;
 		uint8_t ic;
-		if (0 > hci_read_voice_setting(s, &vs, 1000)) {
-			printf("Can't read voice setting on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+		if (hci_read_voice_setting(s, &vs, 1000) < 0) {
+			fprintf(stderr, "Can't read voice setting on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 		vs = htobs(vs);
@@ -627,90 +646,74 @@ void cmd_voice(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_version(int ctl, int hdev, char *opt)
+static void cmd_version(int ctl, int hdev, char *opt)
 {
 	struct hci_version ver;
 	int dd;
 
 	dd = hci_open_dev(hdev);
 	if (dd < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
 	if (hci_read_local_version(dd, &ver, 1000) < 0) {
-		printf("Can't read version info hci%d. %s(%d)\n", 
-			hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't read version info hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
 	print_dev_hdr(&di);
-	printf( "\tHCI Ver: %s (0x%x) HCI Rev: 0x%x LMP Ver: %s (0x%x) LMP Subver: 0x%x\n"
+	printf("\tHCI Ver: %s (0x%x) HCI Rev: 0x%x LMP Ver: %s (0x%x) LMP Subver: 0x%x\n"
 		"\tManufacturer: %s (%d)\n",
-		hci_vertostr(ver.hci_ver), ver.hci_ver, ver.hci_rev, 
-		lmp_vertostr(ver.lmp_ver), ver.lmp_ver, ver.lmp_subver, 
+		hci_vertostr(ver.hci_ver), ver.hci_ver, ver.hci_rev,
+		lmp_vertostr(ver.lmp_ver), ver.lmp_ver, ver.lmp_subver,
 		bt_compidtostr(ver.manufacturer), ver.manufacturer);
 }
 
-void cmd_inq_mode(int ctl, int hdev, char *opt)
+static void cmd_inq_mode(int ctl, int hdev, char *opt)
 {
-	struct hci_request rq;
 	int dd;
 
 	dd = hci_open_dev(hdev);
 	if (dd < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
-	memset(&rq, 0, sizeof(rq));
-
 	if (opt) {
-		write_inquiry_mode_cp cp;
+		uint8_t mode = atoi(opt);
 
-		cp.mode = atoi(opt);
-
-		rq.ogf = OGF_HOST_CTL;
-		rq.ocf = OCF_WRITE_INQUIRY_MODE;
-		rq.cparam = &cp;
-		rq.clen = WRITE_INQUIRY_MODE_RP_SIZE;
-
-		if (hci_send_req(dd, &rq, 1000) < 0) {
-			printf("Can't set inquiry mode on hci%d. %s(%d)\n",
+		if (hci_write_inquiry_mode(dd, mode, 2000) < 0) {
+			fprintf(stderr, "Can't set inquiry mode on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
-		read_inquiry_mode_rp rp;
+		uint8_t mode;
 
-		rq.ogf = OGF_HOST_CTL;
-		rq.ocf = OCF_READ_INQUIRY_MODE;
-		rq.rparam = &rp;
-		rq.rlen = READ_INQUIRY_MODE_RP_SIZE;
-
-		if (hci_send_req(dd, &rq, 1000) < 0) {
-			printf("Can't read inquiry mode on hci%d. %s(%d)\n", 
-							hdev, strerror(errno), errno);
-			exit(1);
-		}
-		if (rp.status) {
-			printf("Read inquiry mode on hci%d returned status %d\n", 
-							hdev, rp.status);
+		if (hci_read_inquiry_mode(dd, &mode, 1000) < 0) {
+			fprintf(stderr, "Can't read inquiry mode on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 
 		print_dev_hdr(&di);
 		printf("\tInquiry mode: %s\n",
-			rp.mode == 1 ? "Inquiry with RSSI" : "Standard Inquiry");
+			mode == 1 ? "Inquiry with RSSI" : "Standard Inquiry");
 	}
 }
 
-void cmd_inq_parms(int ctl, int hdev, char *opt)
+static void cmd_inq_parms(int ctl, int hdev, char *opt)
 {
 	struct hci_request rq;
 	int s;
+
 	if ((s = hci_open_dev(hdev)) < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
@@ -739,8 +742,8 @@ void cmd_inq_parms(int ctl, int hdev, char *opt)
 		if (interval < 0x12 || interval > 0x1000)
 			printf("Warning: inquiry interval out of range!\n");
 
-		if (hci_send_req(s, &rq, 1000) < 0) {
-			printf("Can't set inquiry parameters name on hci%d. %s(%d)\n",
+		if (hci_send_req(s, &rq, 2000) < 0) {
+			fprintf(stderr, "Can't set inquiry parameters name on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
@@ -754,12 +757,12 @@ void cmd_inq_parms(int ctl, int hdev, char *opt)
 		rq.rlen = READ_INQ_ACTIVITY_RP_SIZE;
 
 		if (hci_send_req(s, &rq, 1000) < 0) {
-			printf("Can't read inquiry parameters on hci%d. %s(%d)\n", 
-							hdev, strerror(errno), errno);
+			fprintf(stderr, "Can't read inquiry parameters on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 		if (rp.status) {
-			printf("Read inquiry parameters on hci%d returned status %d\n", 
+			printf("Read inquiry parameters on hci%d returned status %d\n",
 							hdev, rp.status);
 			exit(1);
 		}
@@ -772,12 +775,14 @@ void cmd_inq_parms(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_page_parms(int ctl, int hdev, char *opt)
+static void cmd_page_parms(int ctl, int hdev, char *opt)
 {
 	struct hci_request rq;
 	int s;
+
 	if ((s = hci_open_dev(hdev)) < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
@@ -806,8 +811,8 @@ void cmd_page_parms(int ctl, int hdev, char *opt)
 		if (interval < 0x12 || interval > 0x1000)
 			printf("Warning: page interval out of range!\n");
 
-		if (hci_send_req(s, &rq, 1000) < 0) {
-			printf("Can't set page parameters name on hci%d. %s(%d)\n",
+		if (hci_send_req(s, &rq, 2000) < 0) {
+			fprintf(stderr, "Can't set page parameters name on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
@@ -821,12 +826,12 @@ void cmd_page_parms(int ctl, int hdev, char *opt)
 		rq.rlen = READ_PAGE_ACTIVITY_RP_SIZE;
 
 		if (hci_send_req(s, &rq, 1000) < 0) {
-			printf("Can't read page parameters on hci%d. %s(%d)\n", 
-							hdev, strerror(errno), errno);
+			fprintf(stderr, "Can't read page parameters on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 		if (rp.status) {
-			printf("Read page parameters on hci%d returned status %d\n", 
+			printf("Read page parameters on hci%d returned status %d\n",
 							hdev, rp.status);
 			exit(1);
 		}
@@ -839,12 +844,14 @@ void cmd_page_parms(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_page_to(int ctl, int hdev, char *opt)
+static void cmd_page_to(int ctl, int hdev, char *opt)
 {
 	struct hci_request rq;
 	int s;
+
 	if ((s = hci_open_dev(hdev)) < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
@@ -869,8 +876,8 @@ void cmd_page_to(int ctl, int hdev, char *opt)
 		if (timeout < 0x01 || timeout > 0xFFFF)
 			printf("Warning: page timeout out of range!\n");
 
-		if (hci_send_req(s, &rq, 1000) < 0) {
-			printf("Can't set page timeout on hci%d. %s(%d)\n",
+		if (hci_send_req(s, &rq, 2000) < 0) {
+			fprintf(stderr, "Can't set page timeout on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
@@ -884,12 +891,12 @@ void cmd_page_to(int ctl, int hdev, char *opt)
 		rq.rlen = READ_PAGE_TIMEOUT_RP_SIZE;
 
 		if (hci_send_req(s, &rq, 1000) < 0) {
-			printf("Can't read page timeout on hci%d. %s(%d)\n", 
-							hdev, strerror(errno), errno);
+			fprintf(stderr, "Can't read page timeout on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 		if (rp.status) {
-			printf("Read page timeout on hci%d returned status %d\n", 
+			printf("Read page timeout on hci%d returned status %d\n",
 							hdev, rp.status);
 			exit(1);
 		}
@@ -901,55 +908,36 @@ void cmd_page_to(int ctl, int hdev, char *opt)
 	}
 }
 
-void cmd_afh_mode(int ctl, int hdev, char *opt)
+static void cmd_afh_mode(int ctl, int hdev, char *opt)
 {
-	struct hci_request rq;
 	int dd;
 
 	dd = hci_open_dev(hdev);
 	if (dd < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
-	memset(&rq, 0, sizeof(rq));
-
 	if (opt) {
-		write_afh_mode_cp cp;
+		uint8_t mode = atoi(opt);
 
-		cp.mode = atoi(opt);
-
-		rq.ogf = OGF_HOST_CTL;
-		rq.ocf = OCF_WRITE_AFH_MODE;
-		rq.cparam = &cp;
-		rq.clen = WRITE_AFH_MODE_RP_SIZE;
-
-		if (hci_send_req(dd, &rq, 1000) < 0) {
-			printf("Can't set AFH mode on hci%d. %s(%d)\n",
+		if (hci_write_afh_mode(dd, mode, 2000) < 0) {
+			fprintf(stderr, "Can't set AFH mode on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
 	} else {
-		read_afh_mode_rp rp;
+		uint8_t mode;
 
-		rq.ogf = OGF_HOST_CTL;
-		rq.ocf = OCF_READ_AFH_MODE;
-		rq.rparam = &rp;
-		rq.rlen = READ_AFH_MODE_RP_SIZE;
-
-		if (hci_send_req(dd, &rq, 1000) < 0) {
-			printf("Can't read AFH mode on hci%d. %s(%d)\n", 
-							hdev, strerror(errno), errno);
-			exit(1);
-		}
-		if (rp.status) {
-			printf("Read AFH mode on hci%d returned status %d\n", 
-							hdev, rp.status);
+		if (hci_read_afh_mode(dd, &mode, 1000) < 0) {
+			fprintf(stderr, "Can't read AFH mode on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 			exit(1);
 		}
 
 		print_dev_hdr(&di);
-		printf("\tAFH mode: %s\n", rp.mode == 1 ? "Enabled" : "Disabled");
+		printf("\tAFH mode: %s\n", mode == 1 ? "Enabled" : "Disabled");
 	}
 }
 
@@ -959,7 +947,7 @@ static void print_rev_ericsson(int dd)
 	unsigned char buf[102];
 
 	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = 0x3f;
+	rq.ogf    = OGF_VENDOR_CMD;
 	rq.ocf    = 0x000f;
 	rq.cparam = NULL;
 	rq.clen   = 0;
@@ -967,7 +955,7 @@ static void print_rev_ericsson(int dd)
 	rq.rlen   = sizeof(buf);
 
 	if (hci_send_req(dd, &rq, 1000) < 0) {
-		printf("\n Can't read revision info. %s(%d)\n", strerror(errno), errno);
+		printf("\nCan't read revision info: %s (%d)\n", strerror(errno), errno);
 		return;
 	}
 
@@ -998,6 +986,28 @@ static void print_rev_csr(int dd, uint16_t rev)
 		printf("\tSCO mapping:  %s\n", mapsco ? "PCM" : "HCI");
 }
 
+static void print_rev_digianswer(int dd)
+{
+	struct hci_request rq;
+	unsigned char req[] = { 0x07 };
+	unsigned char buf[102];
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_VENDOR_CMD;
+	rq.ocf    = 0x000e;
+	rq.cparam = req;
+	rq.clen   = sizeof(req);
+	rq.rparam = &buf;
+	rq.rlen   = sizeof(buf);
+
+	if (hci_send_req(dd, &rq, 1000) < 0) {
+		printf("\nCan't read revision info: %s (%d)\n", strerror(errno), errno);
+		return;
+	}
+
+	printf("\t%s\n", buf + 1);
+}
+
 static void print_rev_broadcom(uint16_t hci_rev, uint16_t lmp_subver)
 {
 	printf("\tFirmware %d.%d.%03d\n", hci_rev, lmp_subver >> 8, lmp_subver & 0xff);
@@ -1018,13 +1028,14 @@ static void cmd_revision(int ctl, int hdev, char *opt)
 
 	dd = hci_open_dev(hdev);
 	if (dd < 0) {
-		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		return;
 	}
 
 	if (hci_read_local_version(dd, &ver, 1000) < 0) {
-		printf("Can't read version info hci%d. %s(%d)\n",
-			hdev, strerror(errno), errno);
+		fprintf(stderr, "Can't read version info for hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
 		return;
 	}
 
@@ -1037,6 +1048,9 @@ static void cmd_revision(int ctl, int hdev, char *opt)
 	case 10:
 		print_rev_csr(dd, ver.hci_rev);
 		break;
+	case 12:
+		print_rev_digianswer(dd);
+		break;
 	case 15:
 		print_rev_broadcom(ver.hci_rev, ver.lmp_subver);
 		break;
@@ -1050,7 +1064,7 @@ static void cmd_revision(int ctl, int hdev, char *opt)
 	return;
 }
 
-void print_dev_hdr(struct hci_dev_info *di)
+static void print_dev_hdr(struct hci_dev_info *di)
 {
 	static int hdr = -1;
 	char addr[18];
@@ -1062,12 +1076,12 @@ void print_dev_hdr(struct hci_dev_info *di)
 	ba2str(&di->bdaddr, addr);
 
 	printf("%s:\tType: %s\n", di->name, hci_dtypetostr(di->type) );
-	printf("\tBD Address: %s ACL MTU: %d:%d  SCO MTU: %d:%d\n",
+	printf("\tBD Address: %s ACL MTU: %d:%d SCO MTU: %d:%d\n",
 		addr, di->acl_mtu, di->acl_pkts,
 		di->sco_mtu, di->sco_pkts);
 }
 
-void print_dev_info(int ctl, struct hci_dev_info *di)
+static void print_dev_info(int ctl, struct hci_dev_info *di)
 {
 	struct hci_dev_stats *st = &di->stat;
 
@@ -1097,7 +1111,7 @@ void print_dev_info(int ctl, struct hci_dev_info *di)
 	printf("\n");
 }
 
-struct {
+static struct {
 	char *cmd;
 	void (*func)(int ctl, int hdev, char *opt);
 	char *opt;
@@ -1111,6 +1125,8 @@ struct {
 	{ "noauth",	cmd_auth,	0,		"Disable Authentication" },
 	{ "encrypt",	cmd_encrypt,	0,		"Enable Encryption" },
 	{ "noencrypt",	cmd_encrypt,	0,		"Disable Encryption" },
+	{ "secmgr",	cmd_secmgr,	0,		"Enable Security Manager" },
+	{ "nosecmgr",	cmd_secmgr,	0,		"Disable Security Manager" },
 	{ "piscan",	cmd_scan,	0,		"Enable Page and Inquiry scan" },
 	{ "noscan",	cmd_scan,	0,		"Disable scan" },
 	{ "iscan",	cmd_scan,	0,		"Enable Inquiry scan" },
@@ -1135,7 +1151,7 @@ struct {
 	{ NULL, NULL, 0 }
 };
 
-void usage(void)
+static void usage(void)
 {
 	int i;
 
@@ -1191,13 +1207,13 @@ int main(int argc, char **argv, char **env)
 	di.dev_id = atoi(argv[0] + 3);
 	argc--; argv++;
 
-	if (ioctl(ctl, HCIGETDEVINFO, (void*)&di)) {
+	if (ioctl(ctl, HCIGETDEVINFO, (void *) &di)) {
 		perror("Can't get device info");
 		exit(1);
 	}
 
 	while (argc > 0) {
-		for (i=0; command[i].cmd; i++) {
+		for (i = 0; command[i].cmd; i++) {
 			if (strncmp(command[i].cmd, *argv, 5))
 				continue;
 
