@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hcitool.c,v 1.45 2004/07/25 07:36:18 holtmann Exp $
+ *  $Id: hcitool.c,v 1.54 2005/01/13 22:58:13 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,39 +33,27 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <signal.h>
-#include <fcntl.h>
 #include <errno.h>
-#include <ctype.h>
-
-#include <termios.h>
-#include <fcntl.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <getopt.h>
-#include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 
-extern int optind,opterr,optopt;
-extern char *optarg;
-
 #define for_each_opt(opt, long, short) while ((opt=getopt_long(argc, argv, short ? short:"+", long, NULL)) != -1)
 
 static void usage(void);
 
 static int dev_info(int s, int dev_id, long arg)
 {
-	struct hci_dev_info di = {dev_id: dev_id};
+	struct hci_dev_info di = { dev_id: dev_id };
 	char addr[18];
 
-	if (ioctl(s, HCIGETDEVINFO, (void*) &di))
+	if (ioctl(s, HCIGETDEVINFO, (void *) &di))
 		return 0;
 
 	ba2str(&di.bdaddr, addr);
@@ -91,12 +79,12 @@ static int conn_list(int s, int dev_id, long arg)
 	cl->conn_num = 10;
 	ci = cl->conn_info;
 
-	if (ioctl(s, HCIGETCONNLIST, (void*)cl)) {
+	if (ioctl(s, HCIGETCONNLIST, (void *) cl)) {
 		perror("Can't get connection list");
 		exit(1);
 	}
 
-	for (i=0; i < cl->conn_num; i++, ci++) {
+	for (i = 0; i < cl->conn_num; i++, ci++) {
 		char addr[18];
 		ba2str(&ci->bdaddr, addr);
 		printf("\t%s %s %s handle %d state %d lm %s\n",
@@ -105,6 +93,7 @@ static int conn_list(int s, int dev_id, long arg)
 			addr, ci->handle, ci->state,
 			hci_lmtostr(ci->link_mode));
 	}
+
 	return 0;
 }
 
@@ -122,14 +111,15 @@ static int find_conn(int s, int dev_id, long arg)
 	cl->conn_num = 10;
 	ci = cl->conn_info;
 
-	if (ioctl(s, HCIGETCONNLIST, (void*)cl)) {
+	if (ioctl(s, HCIGETCONNLIST, (void *) cl)) {
 		perror("Can't get connection list");
 		exit(1);
 	}
 
-	for (i=0; i < cl->conn_num; i++, ci++)
-		if (!bacmp((bdaddr_t *)arg, &ci->bdaddr))
+	for (i = 0; i < cl->conn_num; i++, ci++)
+		if (!bacmp((bdaddr_t *) arg, &ci->bdaddr))
 			return 1;
+
 	return 0;
 }
 
@@ -137,7 +127,7 @@ static void hex_dump(char *pref, int width, unsigned char *buf, int len)
 {
 	register int i,n;
 
-	for (i=0, n=1; i<len; i++, n++) {
+	for (i = 0, n = 1; i < len; i++, n++) {
 		if (n == 1)
 			printf("%s", pref);
 		printf("%2.2X ", buf[i]);
@@ -153,11 +143,11 @@ static void hex_dump(char *pref, int width, unsigned char *buf, int len)
 /* Display local devices */
 
 static struct option dev_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{0, 0, 0, 0 }
 };
 
-static char *dev_help = 
+static char *dev_help =
 	"Usage:\n"
 	"\tdev\n";
 
@@ -165,7 +155,7 @@ static void cmd_dev(int dev_id, int argc, char **argv)
 {
 	int opt;
 	for_each_opt(opt, dev_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(dev_help);
 			return;
@@ -179,14 +169,14 @@ static void cmd_dev(int dev_id, int argc, char **argv)
 /* Inquiry */
 
 static struct option inq_options[] = {
-	{"help",	0,0, 'h'},
-	{"length",	1,0, 'l'},
-	{"numrsp",	1,0, 'n'},
-	{"flush",	0,0, 'f'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ "length",	1, 0, 'l' },
+	{ "numrsp",	1, 0, 'n' },
+	{ "flush",	0, 0, 'f' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *inq_help = 
+static char *inq_help =
 	"Usage:\n"
 	"\tinq [--length=N] maximum inquiry duration in 1.28 s units\n"
 	"\t    [--numrsp=N] specify maximum number of inquiry responses\n"
@@ -199,12 +189,12 @@ static void cmd_inq(int dev_id, int argc, char **argv)
 	char addr[18];
 	int i, opt;
 
-	length  = 8;  /* ~10 seconds */
+	length  = 8;	/* ~10 seconds */
 	num_rsp = 100;
-	flags = 0;
+	flags   = 0;
 
 	for_each_opt(opt, inq_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		case 'l':
 			length = atoi(optarg);
 			break;
@@ -244,14 +234,14 @@ static void cmd_inq(int dev_id, int argc, char **argv)
 /* Device scanning */
 
 static struct option scan_options[] = {
-	{"help",	0,0, 'h'},
-	{"length",	1,0, 'l'},
-	{"numrsp",	1,0, 'n'},
-	{"flush",	0,0, 'f'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ "length",	1, 0, 'l' },
+	{ "numrsp",	1, 0, 'n' },
+	{ "flush",	0, 0, 'f' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *scan_help = 
+static char *scan_help =
 	"Usage:\n"
 	"\tscan [--length=N] [--numrsp=N] [--flush]\n";
 
@@ -263,12 +253,12 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 	char name[248];
 	int i, opt, dd;
 
-	length  = 8;  /* ~10 seconds */
+	length  = 8;	/* ~10 seconds */
 	num_rsp = 100;
-	flags = 0;
+	flags   = 0;
 
 	for_each_opt(opt, scan_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		case 'l':
 			length = atoi(optarg);
 			break;
@@ -324,8 +314,8 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 /* Remote name */
 
 static struct option name_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
 static char *name_help =
@@ -339,7 +329,7 @@ static void cmd_name(int dev_id, int argc, char **argv)
 	int opt, dd;
 
 	for_each_opt(opt, name_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(name_help);
 			return;
@@ -373,17 +363,16 @@ static void cmd_name(int dev_id, int argc, char **argv)
 		printf("%s\n", name);
 
 	close(dd);
-
 }
 
 /* Info about remote device */
 
 static struct option info_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *info_help = 
+static char *info_help =
 	"Usage:\n"
 	"\tinfo <bdaddr>\n";
 
@@ -399,7 +388,7 @@ static void cmd_info(int dev_id, int argc, char **argv)
 	int opt, dd, cc = 0;
 
 	for_each_opt(opt, info_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(info_help);
 			return;
@@ -449,7 +438,9 @@ static void cmd_info(int dev_id, int argc, char **argv)
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
 	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
-		if (hci_create_connection(dd, &bdaddr, htobs(di.pkt_type & ACL_PTYPE_MASK), 0, 0x01, &handle, 25000) < 0) {
+		if (hci_create_connection(dd, &bdaddr,
+					htobs(di.pkt_type & ACL_PTYPE_MASK),
+					0, 0x01, &handle, 25000) < 0) {
 			perror("Can't create connection");
 			close(dd);
 			exit(1);
@@ -466,19 +457,24 @@ static void cmd_info(int dev_id, int argc, char **argv)
 	if (hci_read_remote_version(dd, handle, &version, 20000) == 0) {
 		printf("\tLMP Version: %s (0x%x) LMP Subversion: 0x%x\n"
 			"\tManufacturer: %s (%d)\n",
-			lmp_vertostr(version.lmp_ver), version.lmp_ver, version.lmp_subver, 
-			bt_compidtostr(version.manufacturer), version.manufacturer);
+			lmp_vertostr(version.lmp_ver),
+			version.lmp_ver,
+			version.lmp_subver,
+			bt_compidtostr(version.manufacturer),
+			version.manufacturer);
 	}
 
 	if (hci_read_remote_features(dd, handle, features, 20000) == 0) {
 		printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n%s\n",
-				features[0], features[1], features[2], features[3],
-				features[4], features[5], features[6], features[7],
-				lmp_featurestostr(features, "\t\t", 63));
+			features[0], features[1], features[2], features[3],
+			features[4], features[5], features[6], features[7],
+			lmp_featurestostr(features, "\t\t", 63));
 	}
 
-	if (cc)
+	if (cc) {
+		usleep(10000);
 		hci_disconnect(dd, handle, HCI_OE_USER_ENDED_CONNECTION, 10000);
+	}
 
 	close(dd);
 }
@@ -486,11 +482,11 @@ static void cmd_info(int dev_id, int argc, char **argv)
 /* Send arbitrary HCI commands */
 
 static struct option cmd_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *cmd_help = 
+static char *cmd_help =
 	"Usage:\n"
 	"\tcmd <ogf> <ocf> [parameters]\n"
 	"Example:\n"
@@ -506,7 +502,7 @@ static void cmd_cmd(int dev_id, int argc, char **argv)
 	uint8_t ogf;
 
 	for_each_opt(opt, cmd_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(cmd_help);
 			return;
@@ -571,17 +567,16 @@ static void cmd_cmd(int dev_id, int argc, char **argv)
 	hex_dump("  ", 20, ptr, len); fflush(stdout);
 
 	hci_close_dev(dd);
-	return;
 }
 
 /* Display active connections */
 
 static struct option con_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *con_help = 
+static char *con_help =
 	"Usage:\n"
 	"\tcon\n";
 
@@ -590,7 +585,7 @@ static void cmd_con(int dev_id, int argc, char **argv)
 	int opt;
 
 	for_each_opt(opt, con_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(con_help);
 			return;
@@ -604,13 +599,13 @@ static void cmd_con(int dev_id, int argc, char **argv)
 /* Create connection */
 
 static struct option cc_options[] = {
-	{"help",	0,0, 'h'},
-	{"role",	1,0, 'r'},
-	{"ptype",	1,0, 'p'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ "role",	1, 0, 'r' },
+	{ "ptype",	1, 0, 'p' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *cc_help = 
+static char *cc_help =
 	"Usage:\n"
 	"\tcc [--role=m|s] [--ptype=pkt_types] <bdaddr>\n"
 	"Example:\n"
@@ -628,7 +623,7 @@ static void cmd_cc(int dev_id, int argc, char **argv)
 	ptype = HCI_DM1 | HCI_DM3 | HCI_DM5 | HCI_DH1 | HCI_DH3 | HCI_DH5;
 
 	for_each_opt(opt, cc_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		case 'p':
 			hci_strtoptype(optarg, &ptype);
 			break;
@@ -666,7 +661,8 @@ static void cmd_cc(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	if (hci_create_connection(dd, &bdaddr, htobs(ptype), 0, role, &handle, 25000) < 0)
+	if (hci_create_connection(dd, &bdaddr, htobs(ptype),
+					0, role, &handle, 25000) < 0)
 		perror("Can't create connection");
 	hci_close_dev(dd);
 }
@@ -674,11 +670,11 @@ static void cmd_cc(int dev_id, int argc, char **argv)
 /* Close connection */
 
 static struct option dc_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *dc_help = 
+static char *dc_help =
 	"Usage:\n"
 	"\tdc <bdaddr>\n";
 
@@ -689,7 +685,7 @@ static void cmd_dc(int dev_id, int argc, char **argv)
 	int opt, dd;
 
 	for_each_opt(opt, dc_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(dc_help);
 			return;
@@ -730,7 +726,8 @@ static void cmd_dc(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	if (hci_disconnect(dd, htobs(cr->conn_info->handle), HCI_OE_USER_ENDED_CONNECTION, 100) < 0)
+	if (hci_disconnect(dd, htobs(cr->conn_info->handle),
+				HCI_OE_USER_ENDED_CONNECTION, 10000) < 0)
 		perror("Disconnect failed");
 
 	close(dd);
@@ -740,23 +737,22 @@ static void cmd_dc(int dev_id, int argc, char **argv)
 /* Role switch */
 
 static struct option sr_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *sr_help = 
+static char *sr_help =
 	"Usage:\n"
 	"\tsr <bdaddr> <role>\n";
 
 static void cmd_sr(int dev_id, int argc, char **argv)
 {
-	struct hci_request rq;
-	switch_role_cp cp;
-	evt_role_change rp;
+	bdaddr_t bdaddr;
+	uint8_t role;
 	int opt, dd;
 
 	for_each_opt(opt, sr_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(sr_help);
 			return;
@@ -770,21 +766,21 @@ static void cmd_sr(int dev_id, int argc, char **argv)
 		return;
 	}
 
-	str2ba(argv[0], &cp.bdaddr);
+	str2ba(argv[0], &bdaddr);
 	switch (argv[1][0]) {
 	case 'm':
-		cp.role = 0;
+		role = 0;
 		break;
 	case 's':
-		cp.role = 1;
+		role = 1;
 		break;
 	default:
-		cp.role = atoi(argv[1]);
+		role = atoi(argv[1]);
 		break;
 	}
 
 	if (dev_id < 0) {
-		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &cp.bdaddr);
+		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
 		if (dev_id < 0) {
 			fprintf(stderr, "Not connected.\n");
 			exit(1);
@@ -797,50 +793,34 @@ static void cmd_sr(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_LINK_POLICY;
-	rq.ocf    = OCF_SWITCH_ROLE;
-	rq.cparam = &cp;
-	rq.clen   = SWITCH_ROLE_CP_SIZE;
-	rq.rparam = &rp;
-	rq.rlen   = EVT_ROLE_CHANGE_SIZE;
-	rq.event  = EVT_ROLE_CHANGE;
-
-	if (hci_send_req(dd, &rq, 300) < 0) {
+	if (hci_switch_role(dd, &bdaddr, role, 10000) < 0) {
 		perror("Switch role request failed");
 		exit(1);
 	}
 
-	if (rp.status) {
-		fprintf(stderr, "Switch role cmd failed (0x%2.2X)\n", rp.status);
-		exit(1);
-	}
-
 	close(dd);
 }
 
 /* Read RSSI */
 
 static struct option rssi_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *rssi_help = 
+static char *rssi_help =
 	"Usage:\n"
 	"\trssi <bdaddr>\n";
 
 static void cmd_rssi(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	read_rssi_rp rp;
 	bdaddr_t bdaddr;
-	uint16_t handle;
+	int8_t rssi;
 	int opt, dd;
 
 	for_each_opt(opt, rssi_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(rssi_help);
 			return;
@@ -881,26 +861,12 @@ static void cmd_rssi(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	handle = htobs(cr->conn_info->handle);
-
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_STATUS_PARAM;
-	rq.ocf    = OCF_READ_RSSI;
-	rq.cparam = &handle;
-	rq.clen   = 2;
-	rq.rparam = &rp;
-	rq.rlen   = READ_RSSI_RP_SIZE;
-
-	if (hci_send_req(dd, &rq, 100) < 0) {
+	if (hci_read_rssi(dd, htobs(cr->conn_info->handle), &rssi, 1000) < 0) {
 		perror("Read RSSI failed");
 		exit(1);
 	}
 
-	if (rp.status) {
-		printf("Read RSSI returned (error) status 0x%2.2X\n", rp.status);
-		exit(1);
-	}
-	printf("RSSI return value: %d\n", rp.rssi);
+	printf("RSSI return value: %d\n", rssi);
 
 	close(dd);
 	free(cr);
@@ -909,25 +875,23 @@ static void cmd_rssi(int dev_id, int argc, char **argv)
 /* Get link quality */
 
 static struct option lq_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *lq_help = 
+static char *lq_help =
 	"Usage:\n"
 	"\tlq <bdaddr>\n";
 
 static void cmd_lq(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	get_link_quality_rp rp;
 	bdaddr_t bdaddr;
-	uint16_t handle;
+	uint8_t lq;
 	int opt, dd;
 
 	for_each_opt(opt, lq_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(lq_help);
 			return;
@@ -968,27 +932,12 @@ static void cmd_lq(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	handle = htobs(cr->conn_info->handle);
-
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_STATUS_PARAM;
-	rq.ocf    = OCF_GET_LINK_QUALITY;
-	rq.cparam = &handle;
-	rq.clen   = 2;
-	rq.rparam = &rp;
-	rq.rlen   = GET_LINK_QUALITY_RP_SIZE;
-
-	if (hci_send_req(dd, &rq, 100) < 0) {
-		perror("HCI get_link_quality request failed");
+	if (hci_read_link_quality(dd, htobs(cr->conn_info->handle), &lq, 1000) < 0) {
+		perror("HCI read_link_quality request failed");
 		exit(1);
 	}
 
-	if (rp.status) {
-		fprintf(stderr, "HCI get_link_quality cmd failed (0x%2.2X)\n", 
-				rp.status);
-		exit(1);
-	}
-	printf("Link quality: %d\n", rp.link_quality);
+	printf("Link quality: %d\n", lq);
 
 	close(dd);
 	free(cr);
@@ -997,25 +946,23 @@ static void cmd_lq(int dev_id, int argc, char **argv)
 /* Get transmit power level */
 
 static struct option tpl_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *tpl_help = 
+static char *tpl_help =
 	"Usage:\n"
 	"\ttpl <bdaddr> [type]\n";
 
 static void cmd_tpl(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	read_transmit_power_level_cp cp;
-	read_transmit_power_level_rp rp;
 	bdaddr_t bdaddr;
+	uint8_t type, level;
 	int opt, dd;
 
 	for_each_opt(opt, tpl_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(tpl_help);
 			return;
@@ -1030,7 +977,7 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 	}
 
 	str2ba(argv[0], &bdaddr);
-	cp.type = (argc > 1) ? atoi(argv[1]) : 0;
+	type = (argc > 1) ? atoi(argv[1]) : 0;
 
 	if (dev_id < 0) {
 		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
@@ -1056,28 +1003,14 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 		perror("Get connection info failed");
 		exit(1);
 	}
-	cp.handle = htobs(cr->conn_info->handle);
 
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_HOST_CTL;
-	rq.ocf    = OCF_READ_TRANSMIT_POWER_LEVEL;
-	rq.cparam = &cp;
-	rq.clen   = READ_TRANSMIT_POWER_LEVEL_CP_SIZE;
-	rq.rparam = &rp;
-	rq.rlen   = READ_TRANSMIT_POWER_LEVEL_RP_SIZE;
-
-	if (hci_send_req(dd, &rq, 100) < 0) {
+	if (hci_read_transmit_power_level(dd, htobs(cr->conn_info->handle), type, &level, 1000) < 0) {
 		perror("HCI read transmit power level request failed");
 		exit(1);
 	}
 
-	if (rp.status) {
-		fprintf(stderr, "HCI read_transmit_power_level cmd failed (0x%2.2X)\n", 
-				rp.status);
-		exit(1);
-	}
 	printf("%s transmit power level: %d\n",
-		(cp.type == 0) ? "Current" : "Maximum", rp.level);
+		(type == 0) ? "Current" : "Maximum", level);
 
 	close(dd);
 	free(cr);
@@ -1086,25 +1019,24 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 /* Get AFH channel map */
 
 static struct option afh_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *afh_help = 
+static char *afh_help =
 	"Usage:\n"
 	"\tafh <bdaddr>\n";
 
 static void cmd_afh(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	read_afh_map_rp rp;
 	bdaddr_t bdaddr;
 	uint16_t handle;
+	uint8_t mode, map[10];
 	int opt, dd;
 
 	for_each_opt(opt, afh_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(afh_help);
 			return;
@@ -1147,30 +1079,16 @@ static void cmd_afh(int dev_id, int argc, char **argv)
 
 	handle = htobs(cr->conn_info->handle);
 
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_STATUS_PARAM;
-	rq.ocf    = OCF_READ_AFH_MAP;
-	rq.cparam = &handle;
-	rq.clen   = 2;
-	rq.rparam = &rp;
-	rq.rlen   = READ_AFH_MAP_RP_SIZE;
-
-	if (hci_send_req(dd, &rq, 100) < 0) {
+	if (hci_read_afh_map(dd, handle, &mode, map, 1000) < 0) {
 		perror("HCI read AFH map request failed");
 		exit(1);
 	}
 
-	if (rp.status) {
-		fprintf(stderr, "HCI read_afh_map cmd failed (0x%2.2X)\n", 
-				rp.status);
-		exit(1);
-	}
-
-	if (rp.mode == 0x01) {
+	if (mode == 0x01) {
 		int i;
 		printf("AFH map: 0x");
 		for (i = 0; i < 10; i++)
-			printf("%02x", rp.map[i]);
+			printf("%02x", map[i]);
 		printf("\n");
 	} else
 		printf("AFH disabled\n");
@@ -1182,11 +1100,11 @@ static void cmd_afh(int dev_id, int argc, char **argv)
 /* Set connection packet type */
 
 static struct option cpt_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *cpt_help = 
+static char *cpt_help =
 	"Usage:\n"
 	"\tcpt <bdaddr> <packet_types>\n";
 
@@ -1200,7 +1118,7 @@ static void cmd_cpt(int dev_id, int argc, char **argv)
 	int opt, dd, ptype;
 
 	for_each_opt(opt, cpt_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(cpt_help);
 			return;
@@ -1266,26 +1184,23 @@ static void cmd_cpt(int dev_id, int argc, char **argv)
 /* Get/Set link supervision timeout */
 
 static struct option lst_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *lst_help = 
+static char *lst_help =
 	"Usage:\n"
 	"\tlst <bdaddr> [new value in slots]\n";
 
 static void cmd_lst(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	read_link_supervision_timeout_rp rp;
-	write_link_supervision_timeout_cp cp;
 	bdaddr_t bdaddr;
-	uint16_t handle;
+	uint16_t timeout;
 	int opt, dd;
 
 	for_each_opt(opt, lst_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(lst_help);
 			return;
@@ -1326,43 +1241,23 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	handle = htobs(cr->conn_info->handle);
-
 	if (argc == 1) {
-		memset(&rq, 0, sizeof(rq));
-		rq.ogf    = OGF_HOST_CTL;
-		rq.ocf    = OCF_READ_LINK_SUPERVISION_TIMEOUT;
-		rq.cparam = &handle;
-		rq.clen   = 2;
-		rq.rparam = &rp;
-		rq.rlen   = READ_LINK_SUPERVISION_TIMEOUT_RP_SIZE;
-
-		if (hci_send_req(dd, &rq, 100) < 0) {
+		if (hci_read_link_supervision_timeout(dd, htobs(cr->conn_info->handle), &timeout, 1000) < 0) {
 			perror("HCI read_link_supervision_timeout request failed");
 			exit(1);
 		}
 
-		if (rp.status) {
-			fprintf(stderr, "HCI read_link_supervision_timeout failed (0x%2.2X)\n", 
-				rp.status);
-			exit(1);
-		}
-		if (rp.link_sup_to)
+		timeout = btohs(timeout);
+
+		if (timeout)
 			printf("Link supervision timeout: %u slots (%.2f msec)\n",
-				rp.link_sup_to, (float)rp.link_sup_to * 0.625);
+				timeout, (float) timeout * 0.625);
 		else
 			printf("Link supervision timeout never expires\n");
 	} else {
-		cp.handle      = htobs(cr->conn_info->handle);
-		cp.link_sup_to = strtol(argv[1], NULL, 10);
+		timeout = btohs(strtol(argv[1], NULL, 10));
 
-		memset(&rq, 0, sizeof(rq));
-		rq.ogf    = OGF_HOST_CTL;
-		rq.ocf    = OCF_WRITE_LINK_SUPERVISION_TIMEOUT;
-		rq.cparam = &cp;
-		rq.clen   = WRITE_LINK_SUPERVISION_TIMEOUT_CP_SIZE;
-
-		if (hci_send_req(dd, &rq, 100) < 0) {
+		if (hci_write_link_supervision_timeout(dd, htobs(cr->conn_info->handle), timeout, 1000) < 0) {
 			perror("HCI write_link_supervision_timeout request failed");
 			exit(1);
 		}
@@ -1375,25 +1270,22 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 /* Request authentication */
 
 static struct option auth_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *auth_help = 
+static char *auth_help =
 	"Usage:\n"
 	"\tauth <bdaddr>\n";
 
 static void cmd_auth(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	auth_requested_cp cp;
-	evt_auth_complete rp;
 	bdaddr_t bdaddr;
 	int opt, dd;
 
 	for_each_opt(opt, auth_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(auth_help);
 			return;
@@ -1434,18 +1326,7 @@ static void cmd_auth(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	cp.handle = htobs(cr->conn_info->handle);
-
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_LINK_CTL;
-	rq.ocf    = OCF_AUTH_REQUESTED;
-	rq.cparam = &cp;
-	rq.clen   = AUTH_REQUESTED_CP_SIZE;
-	rq.rparam = &rp;
-	rq.rlen   = EVT_AUTH_COMPLETE_SIZE;
-	rq.event  = EVT_AUTH_COMPLETE;
-
-	if (hci_send_req(dd, &rq, 25000) < 0) {
+	if (hci_authenticate_link(dd, htobs(cr->conn_info->handle), 25000) < 0) {
 		perror("HCI authentication request failed");
 		exit(1);
 	}
@@ -1457,25 +1338,23 @@ static void cmd_auth(int dev_id, int argc, char **argv)
 /* Activate encryption */
 
 static struct option enc_options[] = {
-	{"help",	0,0, 'h'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
 };
 
-static char *enc_help = 
+static char *enc_help =
 	"Usage:\n"
 	"\tenc <bdaddr> [encrypt enable]\n";
 
 static void cmd_enc(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
-	struct hci_request rq;
-	set_conn_encrypt_cp cp;
-	evt_encrypt_change rp;
 	bdaddr_t bdaddr;
+	uint8_t encrypt;
 	int opt, dd;
 
 	for_each_opt(opt, enc_options, NULL) {
-		switch(opt) {
+		switch (opt) {
 		default:
 			printf(enc_help);
 			return;
@@ -1516,19 +1395,9 @@ static void cmd_enc(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	cp.handle = htobs(cr->conn_info->handle);
-	cp.encrypt = (argc > 1) ? atoi(argv[1]) : 1;
+	encrypt = (argc > 1) ? atoi(argv[1]) : 1;
 
-	memset(&rq, 0, sizeof(rq));
-	rq.ogf    = OGF_LINK_CTL;
-	rq.ocf    = OCF_SET_CONN_ENCRYPT;
-	rq.cparam = &cp;
-	rq.clen   = SET_CONN_ENCRYPT_CP_SIZE;
-	rq.rparam = &rp;
-	rq.rlen   = EVT_ENCRYPT_CHANGE_SIZE;
-	rq.event  = EVT_ENCRYPT_CHANGE;
-
-	if (hci_send_req(dd, &rq, 25000) < 0) {
+	if (hci_encrypt_link(dd, htobs(cr->conn_info->handle), encrypt, 25000) < 0) {
 		perror("HCI set encryption request failed");
 		exit(1);
 	}
@@ -1537,6 +1406,74 @@ static void cmd_enc(int dev_id, int argc, char **argv)
 	free(cr);
 }
 
+/* Change connection link key */
+
+static struct option key_options[] = {
+	{ "help",	0, 0, 'h' },
+	{ 0, 0, 0, 0 }
+};
+
+static char *key_help =
+	"Usage:\n"
+	"\tkey <bdaddr>\n";
+
+static void cmd_key(int dev_id, int argc, char **argv)
+{
+	struct hci_conn_info_req *cr;
+	bdaddr_t bdaddr;
+	int opt, dd;
+
+	for_each_opt(opt, key_options, NULL) {
+		switch (opt) {
+		default:
+			printf(key_help);
+			return;
+		}
+	}
+	argc -= optind;
+	argv += optind;
+
+	if (argc < 1) {
+		printf(key_help);
+		return;
+	}
+
+	str2ba(argv[0], &bdaddr);
+
+	if (dev_id < 0) {
+		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
+		if (dev_id < 0) {
+			fprintf(stderr, "Not connected.\n");
+			exit(1);
+		}
+	}
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		perror("HCI device open failed");
+		exit(1);
+	}
+
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr)
+		return;
+
+	bacpy(&cr->bdaddr, &bdaddr);
+	cr->type = ACL_LINK;
+	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+		perror("Get connection info failed");
+		exit(1);
+	}
+
+	if (hci_change_link_key(dd, htobs(cr->conn_info->handle), 25000) < 0) {
+		perror("Changing link key failed");
+		exit(1);
+	}
+
+	close(dd);
+	free(cr);
+}
+
 static struct {
 	char *cmd;
 	void (*func)(int dev_id, int argc, char **argv);
@@ -1560,6 +1497,7 @@ static struct {
 	{ "lst",  cmd_lst,  "Set/display link supervision timeout" },
 	{ "auth", cmd_auth, "Request authentication"               },
 	{ "enc",  cmd_enc,  "Set connection encryption"            },
+	{ "key",  cmd_key,  "Change connection link key"           },
 	{ NULL, NULL, 0 }
 };
 
@@ -1574,7 +1512,7 @@ static void usage(void)
 		"\t--help\tDisplay help\n"
 		"\t-i dev\tHCI device\n");
 	printf("Commands:\n");
-	for (i=0; command[i].cmd; i++)
+	for (i = 0; command[i].cmd; i++)
 		printf("\t%-4s\t%s\n", command[i].cmd,
 		command[i].doc);
 	printf("\n"
@@ -1583,9 +1521,9 @@ static void usage(void)
 }
 
 static struct option main_options[] = {
-	{"help",		0,0, 'h'},
-	{"device",		1,0, 'i'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ "device",	1, 0, 'i' },
+	{ 0, 0, 0, 0 }
 };
 
 int main(int argc, char **argv)
@@ -1594,7 +1532,7 @@ int main(int argc, char **argv)
 	bdaddr_t ba;
 
 	while ((opt=getopt_long(argc, argv, "+i:h", main_options, NULL)) != -1) {
-		switch(opt) {
+		switch (opt) {
 		case 'i':
 			dev_id = hci_devid(optarg);
 			if (dev_id < 0) {
@@ -1624,7 +1562,7 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	for (i=0; command[i].cmd; i++) {
+	for (i = 0; command[i].cmd; i++) {
 		if (strncmp(command[i].cmd, argv[0], 3))
 			continue;
 		command[i].func(dev_id, argc, argv);
