@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: l2ping.c,v 1.4 2004/04/28 10:39:47 holtmann Exp $
+ *  $Id: l2ping.c,v 1.7 2004/10/11 10:31:33 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,18 +51,16 @@
 #include <resolv.h>
 #include <netdb.h>
 
-#include <asm/types.h>
-#include <asm/byteorder.h>
-
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/l2cap.h>
 
 /* Defaults */
 bdaddr_t bdaddr;
-int size  = 20;
-int ident = 200;
-int delay = 1;
-int count = -1;
+int size    = 20;
+int ident   = 200;
+int delay   = 1;
+int count   = -1;
+int timeout = 10;
 
 /* Stats */
 int sent_pkt = 0, recv_pkt = 0;
@@ -123,7 +121,7 @@ static void ping(char *svr)
 
 	/* Initialize buffer */
 	for (i = L2CAP_CMD_HDR_SIZE; i < sizeof(buf); i++)
-		buf[i]=(i%40)+'A';
+		buf[i] = (i % 40) + 'A';
 
 	id = ident;
 
@@ -134,7 +132,7 @@ static void ping(char *svr)
 		/* Build command header */
 		cmd->code  = L2CAP_ECHO_REQ;
 		cmd->ident = id;
-		cmd->len   = __cpu_to_le16(size);
+		cmd->len   = htobs(size);
 
 		gettimeofday(&tv_send, NULL);
 
@@ -151,7 +149,7 @@ static void ping(char *svr)
 			register int err;
 
 			pf[0].fd = s; pf[0].events = POLLIN;
-			if ((err = poll(pf, 1, 10*1000)) < 0) {
+			if ((err = poll(pf, 1, timeout * 1000)) < 0) {
 				perror("Poll failed");
 				exit(1);
 			}
@@ -171,7 +169,7 @@ static void ping(char *svr)
 				exit(1);
 			}
 
-			cmd->len = __le16_to_cpu(cmd->len);
+			cmd->len = btohs(cmd->len);
 
 			/* Check for our id */
 			if( cmd->ident != id )
@@ -194,14 +192,16 @@ static void ping(char *svr)
 			gettimeofday(&tv_recv, NULL);
 			timersub(&tv_recv, &tv_send, &tv_diff);
 
-			printf("%d bytes from %s id %d time %.2fms\n", cmd->len, svr, id, tv2fl(tv_diff));
+			printf("%d bytes from %s id %d time %.2fms\n", cmd->len, svr, id - ident, tv2fl(tv_diff));
 
-			if (delay) sleep(delay);
+			if (delay)
+				sleep(delay);
 		} else {
 			printf("no response from %s: id %d\n", svr, id);
 		}
 
-		if (++id > 254) id = ident;
+		if (++id > 254)
+			id = ident;
 	}
 	stat(0);
 }
@@ -210,7 +210,7 @@ static void usage(void)
 {
 	printf("l2ping - L2CAP ping\n");
 	printf("Usage:\n");
-	printf("\tl2ping [-S source addr] [-s size] [-c count] [-f] <bd_addr>\n");
+	printf("\tl2ping [-S source addr] [-s size] [-c count] [-t timeout] [-f] <bd_addr>\n");
 }
 
 extern int optind,opterr,optopt;
@@ -223,7 +223,7 @@ int main(int argc, char *argv[])
 	/* Default options */
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	while ((opt=getopt(argc,argv,"s:c:fS:")) != EOF) {
+	while ((opt=getopt(argc,argv,"s:c:t:fS:")) != EOF) {
 		switch(opt) {
 		case 'f':
 			/* Kinda flood ping */
@@ -234,6 +234,10 @@ int main(int argc, char *argv[])
 			count = atoi(optarg);
 			break;
 
+		case 't':
+			timeout = atoi(optarg);
+			break;
+
 		case 's':
 			size = atoi(optarg);
 			break;
